# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.test import TestCase
from django.utils import translation

from parler.utils.context import switch_language

from allink_core.core_apps.allink_categories.models import AllinkCategory as Category
from allink_core.core_apps.allink_categories.tests import CategoryTestCaseMixin


class TestCategories(CategoryTestCaseMixin, TestCase):
    """Implementation-specific tests"""

    def test_category_slug_creation(self):
        name = "Root Node"
        root = Category.add_root(name=name)
        root.set_current_language("en")
        root.save()
        self.assertEquals(root.slug, "root-node")

    def test_slug_collision(self):
        root = Category.add_root(name="test")
        root.save()
        root = self.reload(root)
        self.assertEquals(root.slug, "test")
        child1 = root.add_child(name="test")
        self.assertEquals(child1.slug, "test-1")
        child2 = root.add_child(name="test")
        self.assertEquals(child2.slug, "test-2")

    def test_str(self):
        root = Category.add_root(name="test")
        root.save()
        self.assertEqual(root.name, str(root))

    def test_delete(self):
        root = Category.add_root(name="test")
        root.save()
        child1 = root.add_child(name="Child 1")
        self.assertIn(child1, root.get_children())
        try:
            root.delete()
        except TypeError:
            self.fail('Deleting a node throws a TypeError.')
        except:
            self.fail('Deleting a node throws an exception.')
        self.assertNotIn(child1, Category.objects.all())

    def test_non_ascii_slug_generation(self):
        """Test slug generation for common non-ASCII types of characters"""
        root = Category.add_root(name="Root Node")
        root.save()
        child1 = root.add_child(name="Germanic umlauts: ä ö ü ß Ä Ö Ü")
        self.assertEquals(child1.slug, "germanic-umlauts-a-o-u-ss-a-o-u")
        child2 = root.add_child(name="Slavic Cyrillic: смачні пляцки")
        self.assertEquals(child2.slug, "slavic-cyrillic-smachni-pliatski")
        child3 = root.add_child(name="Simplified Chinese: 美味蛋糕")
        self.assertEquals(child3.slug, "simplified-chinese-mei-wei-dan-gao")
        # non-ascii only slug
        child4 = root.add_child(name="ß ў 美")
        self.assertEquals(child4.slug, "ss-u-mei")


class TestCategoryTrees(CategoryTestCaseMixin, TestCase):
    """django-treebeard related tests"""

    def test_create_in_mem_category(self):
        name = "Root Node"
        root = Category.add_root(name=name)
        root.set_current_language("en")
        self.assertEquals(root.name, "Root Node")

    def test_create_in_orm_category(self):
        name = "Root Node"
        root = Category.add_root(name=name)
        root.set_current_language("en")
        root.save()
        root = self.reload(root)
        self.assertEquals(root.name, name)

    def test_tree_depth(self):
        a = Category.add_root(name="A")
        b = a.add_child(name="B")
        c = b.add_child(name="C")
        self.assertEqual(c.depth, 3)

    def test_get_children_count(self):
        a = Category.add_root(name="A")
        a.add_child(name="B")
        self.assertEquals(a.get_children_count(), 1)
        a.add_child(name="C")
        a = self.reload(a)
        self.assertEquals(a.get_children_count(), 2)

    def test_get_children(self):
        a = Category.add_root(name="A")
        b = a.add_child(name="B")
        self.assertIn(b, a.get_children())
        c = a.add_child(name="C")
        a = self.reload(a)
        self.assertIn(c, a.get_children())

    def test_get_descendants(self):
        a = Category.add_root(name="A")
        b = a.add_child(name="B")
        c = b.add_child(name="C")
        self.assertIn(c, a.get_descendants())
        d = b.add_child(name='D')
        b = self.reload(b)
        self.assertIn(d, b.get_descendants())

    def test_get_ancestors(self):
        a = Category.add_root(name="A")
        b = a.add_child(name="B")
        c = b.add_child(name="C")
        self.assertIn(a, b.get_ancestors())
        self.assertIn(a, c.get_ancestors())
        d = b.add_child(name='D')
        self.assertIn(a, d.get_ancestors())

    def test_move_category(self):
        a = Category.add_root(name="A")
        b = a.add_child(name="B")
        c = a.add_child(name="C")
        a = self.reload(a)
        b = self.reload(b)
        self.assertEqual(a, c.get_parent())
        self.assertNotEqual(b, c.get_parent())
        c.move(b, "first-child")
        b = self.reload(b)
        c = self.reload(c)
        self.assertEqual(b, c.get_parent())


class TestCategoryParler(CategoryTestCaseMixin, TestCase):
    """django-parler related tests"""

    def test_add_translations(self):
        values = [
            # language code, name, slug
            ('en', "Cheese Omelette", "cheese-omelette"),
            ('de', "Käseomelett", "kaseomelett"),
            ('fr', "Omelette au Fromage", "omelette-au-fromage"),
        ]

        node = None

        # Create the translations
        for lang, name, slug in values:
            if node:
                with switch_language(node, lang):
                    node.name = name
                    node.save()
            else:
                with translation.override(lang):
                    node = Category.add_root(name=name)
                    node.save()

        # Now test that they exist (and didn't obliterate one another)
        for lang, name, slug in values:
            with switch_language(node, lang):
                self.assertEqual(node.name, name)
                self.assertEqual(node.slug, slug)

        # Now test that we gracefully handle languages where there is no
        # translation.
        with switch_language(node, 'it'):
            try:
                node.name
            except:
                self.fail("Translating to an unavailable language should not "
                          "result in an exception.")



# -*- coding: utf-8 -*-
from cms.models.pluginmodel import CMSPlugin
from rest_framework.generics import RetrieveAPIView
from rest_framework.permissions import AllowAny

from rest_framework.renderers import TemplateHTMLRenderer
from django.http import HttpResponse
from cms.models.pagemodel import Page


class CMSPluginAPIView(RetrieveAPIView):
    """
    - this will return the rendered html for a specific plugin
    - it will not render child plugins
    - some plugins need the current_page, so you can pass an id as a GET param
    """
    permission_classes = (AllowAny, )
    lookup_field = 'id'
    queryset = CMSPlugin.objects.all()
    renderer_classes = (TemplateHTMLRenderer,)

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        instance, plugin = self.object.get_plugin_instance()

        from django.template import RequestContext
        from cms.plugin_rendering import ContentRenderer

        renderer = ContentRenderer(request)
        context = RequestContext(request)

        try:
            # if current_page is supplied, we care about the original request
            request.current_page = Page.objects.get(id=request.GET.get('current_page'))
            request.path_info = request.current_page.get_absolute_url(language=request.LANGUAGE_CODE)
        except (KeyError, Page.DoesNotExist):
            pass

        # Avoid errors if plugin require a request object
        # when rendering.
        context['request'] = request
        content = renderer.render_plugin(instance, context)

        response = HttpResponse(content)
        response['X-Robots-Tag'] = 'noindex'
        return response



# -*- coding: utf-8 -*-
from django.urls import path
from django.views.generic import TemplateView


app_name = 'allink_styleguide'

urlpatterns = [
    path('', TemplateView.as_view(template_name='allink_styleguide/styleguide.html'), name='core'),
]



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.html import strip_tags
from django.contrib.postgres.fields import ArrayField
from cms.models.pluginmodel import CMSPlugin

from filer.fields.image import FilerImageField
from filer.fields.file import FilerFileField

from allink_core.core.utils import get_additional_templates
from allink_core.core.models.choices import HORIZONTAL_ALIGNMENT_CHOICES, VERTICAL_ALIGNMENT_CHOICES
from allink_core.core.models.fields import CMSPluginField


class AllinkContentPlugin(CMSPlugin):
    """
    A plugin representing different column-counts
    and special option for rendering its container-content compared to its Content-Plugins background-image.
    """

    COL_1 = 'col-1'
    COL_1_1 = 'col-1-1'
    COL_2_1 = 'col-2-1'
    COL_1_2 = 'col-1-2'
    COL_3 = 'col-3'
    COL_4 = 'col-4'

    # name | verbose_name | column_count
    TEMPLATES = (
        (COL_1, '1 Column', 1, 'col-3-of-3'),
        (COL_1_1, '2 Columns (1:1)', 2, 'col-1-of-2'),
        (COL_2_1, '2 Columns (2:1)', 2, 'col-1-of-3'),
        (COL_1_2, '2 Columns (1:2)', 2, 'col-2-of-3'),
        (COL_3, '3 Columns', 3, 'col-1-of-3'),
        (COL_4, '4 Columns', 4, 'col-1-of-4'),
    )

    # General Fields
    title = models.CharField(
        'Title',
        help_text='The section title',
        max_length=255,
        blank=True,
        null=True
    )
    title_size = models.CharField(
        'Section Title Size',
        max_length=50,
    )
    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50
    )
    container_enabled = models.BooleanField(
        'Activate "container"',
        help_text='If checked, an inner container with a maximum width is added',
        default=True
    )
    inverted_colors_enabled = models.BooleanField(
        'Activate "inverted text colors"',
        help_text=('If checked, the predefined inverted text colors are applied '
                   '(suitable when using a background image/video)'),
        default=False
    )
    overlay_enabled = models.BooleanField(
        'Activate "overlay"',
        help_text=('If checked, a predefined overlay background gradient/color is applied.<br>'
                   '<strong>Important:</strong> Only applied when used in combination with a background '
                   'image/video/color'),
        default=False
    )
    bg_color = models.CharField(
        'Set a predefined background color',
        max_length=50,
        blank=True,
        null=True
    )
    dynamic_height_enabled = models.BooleanField(
        'Activate dynamic height',
        help_text=('If checked, the section\'s height will grow depending on the height of its children.<br>Note: '
                   'This option is being ignored when "full height" or "parallax" are enabled.'),
        default=False
    )
    bg_image_outer_container = FilerImageField(
        verbose_name='Background-Image',
        on_delete=models.PROTECT,
        help_text='Optional: Set a background image for the content section.',
        related_name='%(app_label)s_%(class)s_bg_image',
        blank=True,
        null=True
    )
    # Video related fields
    video_file = FilerFileField(
        verbose_name='Source',
        on_delete=models.PROTECT,
        help_text=('Recommended video settings:<br><br>Format: '
                   'mp4<br>Codec: H.264<br>Target Bitrate: 2 (video loads quick and runs smooth)<br>Audio: '
                   'Not recommended (no audio = smaller file size and less annoyed visitors)<br>File size: '
                   'Dependent of video length. Generally speaking: Less is more.'),
        blank=True,
        null=True,
        related_name='content_video_file'
    )
    video_poster_image = FilerImageField(
        verbose_name='Video Start Image',
        on_delete=models.PROTECT,
        related_name="content_video_poster_image",
        help_text=('This image is displayed while the video is loading. Ideally, use an '
                   '<strong>exact screen capture image</strong> of the very first frame of the video for best '
                   'results.'),
        blank=True,
        null=True
    )
    video_mobile_image = FilerImageField(
        verbose_name='Mobile Image',
        on_delete=models.PROTECT,
        related_name="content_video_mobile_image",
        help_text='The image that is being displayed on mobile devices instead of the video.',
        blank=True,
        null=True
    )
    anchor = models.CharField(
        verbose_name='ID',
        max_length=255,
        blank=True,
        help_text=('ID of this content section which can be used for anchor reference from links.<br>'
                   'Note: Only letters, numbers and hyphen. No spaces or special chars.'),
    )
    ignore_in_pdf = models.BooleanField(
        'Ignore for pdf export',
        help_text='If checked, this content section will be ignored when generting a pdf.',
        default=False
    )
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )

    cmsplugin_ptr = CMSPluginField()

    project_css_spacings_top_bottom = models.CharField(
        'Spacings',
        help_text='Choose a spacing (top and bottom).',
        max_length=50,
        blank=True,
        null=True
    )
    project_css_spacings_top = models.CharField(
        'Spacings top',
        help_text='Choose a top spacing.',
        max_length=50,
        blank=True,
        null=True
    )
    project_css_spacings_bottom = models.CharField(
        'Spacings bottom',
        help_text='Choose a bottom spacing.',
        max_length=50,
        blank=True,
        null=True
    )

    def __str__(self):
        return str(self.id)

    @classmethod
    def get_templates(cls):
        templates = cls.TEMPLATES
        for a, b, c, d in get_additional_templates('ADDITIONAL_CONTENT'):
            templates += ((a, b, c, d),)
        return templates

    @classmethod
    def get_additional_templates(cls):
        templates = ()
        for a, b, c, d in get_additional_templates('ADDITIONAL_CONTENT'):
            templates += ((a, b, c, d),)
        return templates

    @classmethod
    def get_template_choices(cls):
        choices = ()
        standard = ()
        project = ()

        for template in cls.TEMPLATES:
            standard += ((template[0], template[1]),)

        for template in cls.get_additional_templates():
            project += ((template[0], template[1]),)

        if project:
            choices += (('Standard', standard,),)
            choices += (('Project', project,),)
        else:
            choices = standard

        return choices

    @classmethod
    def get_template_column_count(cls, name):
        for template in cls.get_templates():
            if template[0] == name:
                return template[2]

    def get_short_description(self):
        """
         for better overview in structure mode
         display title, if supplied. if not supplied either display the first title of the first child plugin.
         If the first child has no title and there is a Text Plugin, display the first character of the TextPlugin.
        """
        if self.title:
            return'{} ({})'.format(self.title, self.template)
        else:
            if self.child_plugin_instances:
                for column in self.child_plugin_instances:
                    if column.child_plugin_instances:
                        for plugin in column.child_plugin_instances:
                            if hasattr(plugin, 'title') and plugin.title:
                                return'... {} ({})'.format(plugin.title, self.template)
                            elif plugin.plugin_type == 'TextPlugin':
                                return'... {} ({}) ...'.format(strip_tags(plugin.body)[0:50], self.template)
                            elif plugin.plugin_type == 'CMSAllinkImagePlugin':
                                return'... {} ({}) ...'.format(plugin.picture, self.template)
                            else:
                                return'({})'.format(self.template)
                    else:
                        return'({})'.format(self.template)
            else:
                return'({})'.format(self.template)

    def clean(self):
        if self.video_file and self.video_file.extension not in settings.ALLOWED_VIDEO_EXTENSIONS:
            raise ValidationError(
                'Incorrect file type: %(value)s',
                params={'value': self.video_file.extension},
            )

    @property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        css_classes.append('first') if self.position == 0 else None
        css_classes.append("container-enabled") if self.container_enabled else None
        css_classes.append('section-heading-{}'.format(self.title_size)) if self.title_size else None
        css_classes.append("has-bg-color") if self.bg_color else None
        css_classes.append(self.bg_color) if self.bg_color else None
        css_classes.append("has-bg-video") \
            if self.video_file and self.video_poster_image and self.video_mobile_image else None
        # video is stronger than the image
        css_classes.append("has-bg-image") if self.bg_image_outer_container and not self.video_file else None
        css_classes.append("inverted-colors") if self.inverted_colors_enabled else None
        css_classes.append("overlay-enabled") if self.overlay_enabled else None
        css_classes.append("dynamic-height-enabled") \
            if self.dynamic_height_enabled else None
        css_classes.append("dynamic-height-enabled") if self.dynamic_height_enabled else None
        return ' '.join(css_classes)

    @property
    def css_section_classes(self):
        css_classes = []
        if self.project_css_spacings_top:
            css_classes.append('{}-top'.format(self.project_css_spacings_top))
        if self.project_css_spacings_bottom:
            css_classes.append('{}-bottom'.format(self.project_css_spacings_bottom))
        if self.project_css_spacings_top_bottom:
            css_classes = self.project_css_spacings_top_bottom
            return css_classes
        return ' '.join(css_classes)

    @property
    def attributes(self):
        attributes = []
        return ' '.join(attributes)


class AllinkContentColumnPlugin(CMSPlugin):
    title = models.CharField(
        'Title',
        max_length=255,
        blank=True,
        null=True
    )
    order_mobile = models.IntegerField(
        'Order Mobile',
        help_text=('Some columns should be ordered differently on mobile devices when columns are '
                   'stacked vertically. This option allows you to define the position of the this column.<br><br>'
                   'Note: Columns ordering is ascending (lowest number displayed first)'),
        blank=True,
        null=True
    )
    alignment_horizontal_mobile = models.CharField(
        'Alignment horizontal mobile',
        max_length=50,
        choices=HORIZONTAL_ALIGNMENT_CHOICES,
        help_text='This option overrides the projects default alignment for mobile. (Usually "left")',
        blank=True,
        null=True
    )
    alignment_horizontal_desktop = models.CharField(
        'Alignment horizontal desktop',
        max_length=50,
        choices=HORIZONTAL_ALIGNMENT_CHOICES,
        help_text='This option overrides the projects default alignment for desktop. (Usually "left")',
        blank=True,
        null=True
    )
    alignment_vertical_desktop = models.CharField(
        'Alignment vertical desktop',
        max_length=50,
        choices=VERTICAL_ALIGNMENT_CHOICES,
        help_text='This option overrides the projects default alignment for desktop. (Usually "top")',
        blank=True,
        null=True
    )

    def __str__(self):
        if self.title and self.template:
            return '{} ({})'.format(self.title, self.template)
        else:
            return '({})'.format(self.template)

    @property
    def css_classes(self):
        css_classes = []
        css_classes.append('col-empty') if self.num_children() == 0 else None
        css_classes.append('align-v-desktop-{}'.format(self.alignment_vertical_desktop)) \
            if self.alignment_vertical_desktop else None
        css_classes.append('align-h-desktop-{}'.format(self.alignment_horizontal_desktop)) \
            if self.alignment_horizontal_desktop else None
        css_classes.append('align-h-mobile-{}'.format(self.alignment_horizontal_mobile)) \
            if self.alignment_horizontal_mobile else None
        css_classes.append('col-order-mobile-{}'.format(self.order_mobile))
        return ' '.join(css_classes)

    @property
    def template(self):
        try:
            return self.parent.allink_content_allinkcontentplugin.template
        except AttributeError:
            # a AllinkContentColumnPlugin should always have a parent and therefore a template.
            # But when copied, the parent isn't set yet.
            return 'no template'



# -*- coding: utf-8 -*-
import urllib.parse
from django.conf import settings
from django.db import models
from django.db.models import Q, QuerySet
from django.contrib.postgres.fields import ArrayField
from django.utils.functional import cached_property
from cms.models.pluginmodel import CMSPlugin
from djangocms_text_ckeditor.fields import HTMLField
from allink_core.core.utils import get_additional_templates
from allink_core.core.models.fields import CMSPluginField
from allink_core.core_apps.allink_categories.models import AllinkCategory

__all__ = [
    'AllinkBaseAppContentPlugin',
    'AllinkBaseFormPlugin',
    'AllinkBaseSearchPlugin',
]


class AllinkBaseAppContentPlugin(CMSPlugin):
    """
    TODO
    Base plugin which provides standard functionality
    all Content App-Plugins should inherit from this, to create a "app pointer plugin"

    - allows to display application content for different apps
    - ability to filter and sort entries
    - manually select entries (search entries and select/ sort)

    """

    data_model = None

    # PAGINATION
    NO = 'no'
    LOAD = 'load'
    LOAD_REST = 'load_rest'
    # PAGES = 'pages'

    PAGINATION_TYPE = (
        (NO, 'None'),
        (LOAD, 'Add "Load more"-Button'),
        (LOAD_REST, 'Add "Load all"-Button'),
        # (PAGES, 'Page Navigation'),
    )

    # COLUMN AMOUNT
    COLUMN_AMOUNT = (
        (1, 1),
        (2, 2),
        (3, 3),
        (4, 4),
    )

    # ORDERING
    DEFAULT = 'default'
    TITLE_ASC = 'title_asc'
    TITLE_DESC = 'title_desc'
    LATEST = 'latest'
    EARLIEST = 'earliest'
    CATEGORY = 'category'

    ORDERING = (
        (DEFAULT, '---------'),
        (TITLE_ASC, 'A-Z (title)'),
        (TITLE_DESC, 'Z-A (title'),
        (LATEST, 'latest first'),
        (EARLIEST, 'earliest first'),
        (CATEGORY, 'category ordering'),
    )

    FILTERING = (
        (DEFAULT, '---------'),
    )

    # FIELDS
    categories = models.ManyToManyField(
        AllinkCategory,
        blank=True
    )
    categories_and = models.ManyToManyField(
        AllinkCategory,
        blank=True,
        related_name='%(app_label)s_%(class)s_categories_and'
    )

    manual_filtering = models.CharField(
        max_length=50,
        null=True,
        blank=True
    )

    manual_ordering = models.CharField(
        max_length=50,
        null=True,
        blank=True
    )

    # manual_entries  -> defined in subclasses (no elegant way found to define this here.)
    # apphook_page -> defined in subclasses (no elegant way found to define this here.)

    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50
    )
    category_navigation_enabled = models.BooleanField(
        'Show category navigation',
        help_text=
        'If checked, a filter navigation with all selected categories is displayed.'
        '<br>Please note: A category is only displayed if it contains items.',
        default=False
    )
    category_navigation_all = models.BooleanField(
        'Show category "all"',
        help_text='If checked, a category "all" in filter navigation is displayed.',
        default=False
    )
    category_navigation = models.ManyToManyField(
        AllinkCategory,
        related_name='%(app_label)s_%(class)s_category_navigation',
        verbose_name='Categories for Navigation',
        help_text=
        'You can explicitly define the categories for the category navigation here.'
        ' This will override the automatically set of categories'
        ' (either the one generated from "Filter & Ordering" or "Manual entries")',
        blank=True,
    )
    softpage_enabled = models.BooleanField(
        'Show detailed information in Softpage',
        help_text=
        'If checked, the detail view of an entry will be displayed in a "softpage".'
        ' Otherwise the page will be reloaded.',
        default=True
    )
    detail_link_enabled = models.BooleanField(
        'Show detail link',
        help_text='If checked, a link/button to the detail view will be displayed.',
        default=True
    )
    items_per_row = models.IntegerField(
        'Grid items per row',
        help_text='Only applied if a "Grid" template has been selected.',
        choices=COLUMN_AMOUNT,
        default=3
    )
    paginated_by = models.IntegerField(
        'Max. entries per page',
        default=0,
        help_text=
        'Limit the number of entries (in case of the "load more" pagination type: entries per page).'
        ' Default is "0" (show all entries)'
    )
    pagination_type = models.CharField(
        'Pagination Type',
        max_length=50,
        choices=PAGINATION_TYPE,
        default=PAGINATION_TYPE[0]
    )
    load_more_button_text = models.CharField(
        'Text for "Load .."-Button',
        help_text=
        'If left blank, a default text will be used. <br>Note: Should the default text be adjusted site-wide,'
        ' please contact the project manager (such changes can be made on a code level)',
        max_length=255,
        null=True,
        blank=True
    )
    detail_link_text = models.CharField(
        'Text for "Detail"-Link',
        help_text=
        'If left blank, a default text will be used.<br>Note: Should the default text be adjusted site-wide,'
        ' please contact the project manager (such changes can be made on a code level)',
        max_length=255,
        null=True,
        blank=True
    )
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )
    cmsplugin_ptr = CMSPluginField()

    class Meta:
        abstract = True

    def __str__(self):
        return str(self.pk)

    @classmethod
    def get_templates(cls):
        templates = ()
        for x, y in get_additional_templates(cls.data_model._meta.model_name):
            templates += ((x, y),)
        return templates

    @classmethod
    def get_filtering_choices(cls):
        return cls.FILTERING

    @classmethod
    def get_ordering_choices(cls):
        return cls.ORDERING

    @cached_property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        css_classes.append('{}-template'.format(self.template)) if self.template else None
        css_classes.append('items-per-row-{}'.format(self.items_per_row)) if self.items_per_row else None
        return ' '.join(css_classes)

    def copy_relations(self, oldinstance):
        for i in oldinstance.categories.all():
            self.categories.add(i)
        for i in oldinstance.categories_and.all():
            self.categories_and.add(i)
        for i in oldinstance.category_navigation.all():
            self.category_navigation.add(i)
        for i in oldinstance.manual_entries.all():
            self.manual_entries.add(i)

    def get_model_name(self):
        return self.data_model._meta.model_name

    def get_app_can_have_categories(self):
        if self.data_model._meta.model_name in dict(settings.PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES):
            return True
        else:
            return False

    def get_correct_template(self, file):
        # file can only be '_items', 'content', 'no_results'
        if file != 'no_results':
            template = '{}/plugins/{}/{}.html'.format(self.data_model._meta.app_label, self.template, file)
        else:
            template = '{}/plugins/{}.html'.format(self.data_model._meta.app_label, file)
        return template

    @cached_property
    def fetch_categories(self):
        return self.categories.all()

    @cached_property
    def fetch_first_category(self):
        return self.categories.first()

    @cached_property
    def fetch_categories_and(self):
        return self.categories_and.all()

    @cached_property
    def fetch_manual_entries(self):
        return self.manual_entries.active()

    @cached_property
    def fetch_category_navigation(self):
        category_navigation = []
        # if manual entries are selected the category navigation
        # is created from all distinct categories in selected entries
        if self.fetch_manual_entries:
            for entry in self.fetch_manual_entries:
                for category in entry.fetch_categories:
                    if category not in category_navigation:
                        category_navigation.append(category)
        else:
            # override auto category nav
            if self.category_navigation.exists():
                for category in self.category_navigation.all():
                    if isinstance(self.get_render_queryset_for_display(category), QuerySet):
                        if self.get_render_queryset_for_display(category):
                            category_navigation.append(category)
                    else:
                        if len(self.get_render_queryset_for_display(category)):
                            category_navigation.append(category)
            # auto category nav
            else:
                if self.fetch_categories:
                    for category in self.fetch_categories:
                        if len(self.get_render_queryset_for_display(category)) or \
                                self.get_render_queryset_for_display(category):
                            category_navigation.append(category)
                # auto category nav, if no categories are specified
                else:
                    from allink_core.core_apps.allink_categories.models import AllinkCategory
                    categories = self.get_render_queryset_for_display().filter(~Q(categories=None)).values_list(
                        'categories')
                    category_navigation = list(AllinkCategory.objects.filter(id__in=categories).distinct())
        return category_navigation

    def _apply_filtering_to_queryset_for_display(self, queryset):
        """
        applies individual query filters on given queryset.
        override in app instance (i.e. events) for custom filters
        see allink_core/apps/events/abstract_models.py for reference
        """
        return queryset

    def _apply_ordering_to_queryset_for_display(self, queryset):
        # latest
        if self.manual_ordering == AllinkBaseAppContentPlugin.LATEST:
            return queryset.latest()
        # earliest
        elif self.manual_ordering == AllinkBaseAppContentPlugin.EARLIEST:
            return queryset.earliest()
        # A-Z
        elif self.manual_ordering == AllinkBaseAppContentPlugin.TITLE_ASC:
            return queryset.title_asc()
        # Z-A
        elif self.manual_ordering == AllinkBaseAppContentPlugin.TITLE_DESC:
            return queryset.title_desc()
        # category
        elif self.manual_ordering == AllinkBaseAppContentPlugin.CATEGORY:
            # return queryset.category()
            # https://code.djangoproject.com/ticket/24218
            distinct_entries = []
            for entry in queryset.category():
                if entry not in distinct_entries:
                    distinct_entries.append(entry)
            return distinct_entries
        else:
            return queryset.distinct()

    def get_render_queryset_for_display(self, category=None):
        """
         returns all data_model objects distinct to id which are in the selected categories
          - category: category instance
            -> adds additional query

        after refactoring:
        - category will be supplied via filters, request will be removed
        """
        # manual entries
        if self.fetch_manual_entries:
            apply_ordering = False
            queryset = self.fetch_manual_entries
        else:
            queryset = self.data_model.objects.active()

        if self.fetch_categories or category:
            if category:
                queryset = queryset.filter_by_category(category)
            else:
                queryset = queryset.filter_by_categories(categories=self.fetch_categories)

            if self.fetch_categories_and:
                queryset = queryset.filter_by_categories(categories=self.fetch_categories_and)

        # apply filtering
        queryset = self._apply_filtering_to_queryset_for_display(queryset)

        # apply ordering
        # if apply_ordering:
        queryset = self._apply_ordering_to_queryset_for_display(queryset)

        # hook for prefetching related
        queryset = self._get_queryset_with_prefetch_related(queryset)

        return queryset

    def _get_queryset_with_prefetch_related(self, ordered_qs):
        if type(ordered_qs) is list:
            return ordered_qs
        return ordered_qs.prefetch_related('translations', 'preview_image')


class AllinkBaseSearchPlugin(CMSPlugin):
    data_model = None

    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50,
        default='search_grid_static'
    )

    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )

    cmsplugin_ptr = CMSPluginField()

    class Meta:
        abstract = True

    def __str__(self):
        return str(self.pk)

    def get_model_name(self):
        return self.data_model._meta.model_name

    @cached_property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        return ' '.join(css_classes)

    @classmethod
    def get_templates(cls):
        templates = ()
        for x, y in get_additional_templates('{}_SEARCH'.format(cls.data_model._meta.model_name)):
            templates += ((x, y),)
        return templates


class AllinkBaseFormPlugin(CMSPlugin):
    form_text = HTMLField(
        'Form text',
        blank=True,
        help_text='This text will be shown, before the actual form fields and later replaced with the success message.'
    )
    email_subject = models.CharField(
        'External email subject',
        max_length=255,
        blank=True,
    )

    from_email_address = models.EmailField(
        'From e-mail address',
        default=settings.DEFAULT_FROM_EMAIL,
    )

    internal_recipients = ArrayField(
        models.EmailField(
            blank=True,
        ),
        blank=True,
        null=True,
        verbose_name='Internal e-mail recipients',
    )

    success_message = HTMLField(
        'Success message',
        blank=True,
        help_text='This text will be shown, after form completion.'
    )

    cmsplugin_ptr = CMSPluginField()

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-

from django.urls import reverse

from cms.toolbar.items import Break
from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER, ADMIN_SITES_BREAK
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar
from cms.extensions.toolbar import ExtensionToolbar

from allink_core.core.loading import get_model

Config = get_model('config', 'Config')

AllinkPageExtension = get_model('config', 'AllinkPageExtension')
AllinkTitleExtension = get_model('config', 'AllinkTitleExtension')


@toolbar_pool.register
class ConfigToolbar(CMSToolbar):
    model = Config

    def populate(self):
        admin_menu = self.toolbar.get_menu(ADMIN_MENU_IDENTIFIER)
        position = admin_menu.find_first(Break, identifier=ADMIN_SITES_BREAK)
        allink_menu = admin_menu.get_or_create_menu(
            'allink-menu',
            'allink',
            position=position
        )
        url = reverse('admin:{}_{}_changelist'.format(self.model._meta.app_label, self.model._meta.model_name))
        allink_menu.add_sideframe_item('Config', url=url)


class AllinkPageExtensionToolbar(ExtensionToolbar):
    model = AllinkPageExtension

    def populate(self):
        current_page_menu = self._setup_extension_toolbar()
        if current_page_menu and self.toolbar.edit_mode_active:
            position = 5
            sub_menu = self._get_sub_menu(current_page_menu, 'submenu_label', 'Meta settings', position)
            page_extension, url = self.get_page_extension_admin()
            if url:
                sub_menu.add_modal_item('Images', url=url,
                                        disabled=not self.toolbar.edit_mode_active)


toolbar_pool.register(AllinkPageExtensionToolbar)


class AllinkTitleExtensionToolbar(ExtensionToolbar):
    model = AllinkTitleExtension

    def populate(self):
        current_page_menu = self._setup_extension_toolbar()
        if current_page_menu and self.toolbar.edit_mode_active:
            position = 5
            sub_menu = self._get_sub_menu(current_page_menu, 'submenu_label', 'Meta settings', position)
            from django.conf import settings
            for language_code, _ in settings.LANGUAGES:
                title_extension, url = self.get_title_extension_admin(language_code)[0]
                sub_menu.add_modal_item('Text {}'.format(language_code), url=url, disabled=not self.toolbar.edit_mode_active)


toolbar_pool.register(AllinkTitleExtensionToolbar)



import os

__version__ = '2.7.2'

# Cheeky setting that allows each template to be accessible by two paths.
# Eg: the template 'allink/templates/allink/base.html' can be accessed via both
# 'base.html' and 'allink/base.html'.  This allows allink's templates to be
# extended by templates with the same filename


ALLINK_CORE_MAIN_TEMPLATE_DIRS = [
    os.path.join(os.path.dirname(os.path.abspath(__file__)), 'core/templates/allink_core'),
]

ALLINK_CORE_ALLINK_APPS = [
    'allink_core.apps.config',
    'allink_core.apps.locations',
    'allink_core.apps.news',
    'allink_core.apps.people',
]


def get_core_apps(overrides=None, installed=None):
    """
    Return a list of allink apps amended with any passed overrides
    """
    installed_apps = []
    if not installed:
        installed_apps = ALLINK_CORE_ALLINK_APPS
    else:
        for app in installed:
            if app in ALLINK_CORE_ALLINK_APPS:
                installed_apps.append(app)

    if not overrides:
        return installed_apps

    # Conservative import to ensure that this file can be loaded
    # without the presence Django.
    from django.utils import six
    if isinstance(overrides, six.string_types):
        raise ValueError(
            "get_core_apps expects a list or tuple of apps "
            "to override")

    def get_app_label(app_label, overrides):
        pattern = app_label.replace('allink_core.apps.', '')
        for override in overrides:
            if override.endswith(pattern):
                return override
        return app_label

    apps = []
    for app_label in installed_apps:
        apps.append(get_app_label(app_label, overrides))
    return apps



# -*- coding: utf-8 -*-
from django import forms
from cms.models.pagemodel import Page
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from allink_core.core.models.mixins import AllinkTeaserMixin
from allink_core.apps.config.utils import get_page_teaser_dict
from allink_core.core.forms.fields import SelectLinkField
from allink_core.core.forms.mixins import AllinkInternalLinkFieldMixin
from allink_core.core_apps.allink_teaser.models import AllinkTeaserPlugin
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AllinkTeaserPluginForm(AllinkInternalLinkFieldMixin, forms.ModelForm):

    internal_link = SelectLinkField(label='Link Internal')

    class Meta:
        model = AllinkTeaserPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def __init__(self, *args, **kwargs):
        super(AllinkTeaserPluginForm, self).__init__(*args, **kwargs)
        self.fields['template'] = forms.CharField(
            label='Template',
            widget=forms.Select(choices=self.instance.get_templates()),
            required=True,
        )


@plugin_pool.register_plugin
class CMSAllinkTeaserPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkTeaserPlugin
    name = 'Teaser'
    module = 'Generic'
    form = AllinkTeaserPluginForm

    fieldsets = (
        (None, {
            'fields': (
                'template',
                'internal_link',
            ),
        }),
        ('Override teaser content', {
            'classes': ('collapse',),
            'fields': (
                'teaser_image',
                'teaser_title',
                'teaser_technical_title',
                'teaser_description',
                'teaser_link_text',
                'softpage_enabled',
            )
        })
    )

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_teaser/{}/item.html'.format(instance.template)
        return template

    def render(self, context, instance, placeholder):
        context = super(CMSAllinkTeaserPlugin, self).render(context, instance, placeholder)
        context.update(self.get_teaser_context(instance))
        return context

    def get_teaser_override_fields(self, instance, teaser_dict):
        for field in filter(lambda x: x.name.startswith('teaser_'), instance._meta.get_fields()):
            if getattr(instance, field.name):
                teaser_dict[field.name] = getattr(instance, field.name)

    def get_teaser_context(self, instance):
        teaser_dict = dict()

        # check if link object is a page
        if issubclass(instance.link_object.__class__, Page):
            teaser_dict = get_page_teaser_dict(instance.link_object)
        # check if link object is a app detail link
        elif issubclass(instance.link_object.__class__, AllinkTeaserMixin):
            teaser_dict = instance.link_object.teaser_dict

        # override fields from teaser plugin
        self.get_teaser_override_fields(instance, teaser_dict)

        return teaser_dict



# -*- coding: utf-8 -*-
import ast
from django import template

from allink_core.apps.config.utils import get_page_meta_dict

register = template.Library()


@register.inclusion_tag('templatetags/allink_meta_og.html', takes_context=True)
def render_meta_og(context, obj=None, overwrite_dict=None):
    """
    e.g:
    {% render_meta_og %} or with an object {% render_meta_og news_entry %}

    :param context:
    default param for inclusion tag
    :param obj:
    an object which at least defines the required meta_ fields (defined in AllinkMetaTagMixin)
    :param overwrite_dict:
    you can overwrite or supply all the keys used in the template as a dict represented as string.
    e.g: {% render_meta_og overwrite_dict="{'meta_title': 'Hello', 'meta_description': 'Some Text',
    'meta_image_url':'www.foo.ch/image.jpg'}" %}
    Valid context variables are:
        meta_page_title
        meta_description
        google_site_verification
        meta_title
        meta_image_url
    :return:
    the updated context with all meta_ fields
    """

    meta_context = dict()
    if obj:
        meta_context = obj.meta_dict
    elif getattr(context.request, 'current_page', None):
        meta_context = get_page_meta_dict(getattr(context.request, 'current_page'))

    if overwrite_dict:
        overwrite_dict = ast.literal_eval(overwrite_dict)
        for key, value in overwrite_dict.items():
            meta_context[key] = value

    context.update(meta_context)
    return context



# -*- coding: utf-8 -*-
from django import forms
from django.utils.html import strip_tags

from django.conf import settings
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from webpack_loader.utils import get_files
from allink_core.core.utils import get_ratio_choices, get_additional_choices
from allink_core.core_apps.allink_gallery.models import AllinkGalleryPlugin, AllinkGalleryImagePlugin
from allink_core.core.loading import get_model
from allink_core.core.admin.mixins import AllinkMediaAdminMixin

Config = get_model('config', 'Config')


class AllinkGalleryPluginForm(forms.ModelForm):

    class Meta:
        model = AllinkGalleryPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def __init__(self, *args, **kwargs):
        super(AllinkGalleryPluginForm, self).__init__(*args, **kwargs)
        self.fields['ratio'] = forms.CharField(
            label='Ratio',
            help_text='This option overrides the default settings for the gallery plugin.',
            widget=forms.Select(choices=get_ratio_choices()),
            required=False,
        )
        self.fields['template'] = forms.CharField(
            label='Template',
            widget=forms.Select(choices=self.instance.get_templates()),
            required=True,
        )
        if get_additional_choices('GALLERY_CSS_CLASSES'):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predifined variations',
                choices=get_additional_choices('GALLERY_CSS_CLASSES'),
                required=False,
            )


class AllinkGalleryImagePluginForm(forms.ModelForm):

    class Meta:
        model = AllinkGalleryImagePlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def clean(self):
        cleaned_data = super(AllinkGalleryImagePluginForm, self).clean()
        form_data = self.cleaned_data
        text_length = len(strip_tags(form_data['text']))
        max_length = settings.ALLINK_GALLERY_PLUGIN_CAPTION_TEXT_MAX_LENGTH

        if max_length and text_length > max_length:
            self.add_error(
                'text', 'There are only {} characters allowed in text field. Currently there are {} characters.'
                .format(max_length, text_length))

        return cleaned_data


@plugin_pool.register_plugin
class CMSAllinkGalleryPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkGalleryPlugin
    name = 'Gallery'
    module = 'Generic'
    allow_children = True
    child_classes = ['CMSAllinkGalleryImagePlugin']
    form = AllinkGalleryPluginForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'template',
                    'folder',
                ),
            }),
            ('Slider Options', {
                'fields': [
                    'ratio',
                    'fullscreen_enabled',
                    'counter_enabled',
                    'auto_start_enabled',
                ]
            }),
            ('Advanced Options', {
                'classes': ('collapse',),
                'fields': (
                    'project_css_classes',
                )
            })
        )

        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_gallery/{}/content.html'.format(instance.template)
        return template


@plugin_pool.register_plugin
class CMSAllinkGalleryImagePlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkGalleryImagePlugin
    name = 'Gallery Image'
    module = 'Generic'
    allow_children = False
    form = AllinkGalleryImagePluginForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'image',
                    'title',
                    'text',
                ),
            }),
        )

        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_gallery/{}/item.html'.format(instance.template)
        return template



# -*- coding: utf-8 -*-
from menus.base import NavigationNode
from menus.menu_pool import menu_pool
from cms.menu_bases import CMSAttachMenu

from allink_core.core.loading import get_model, get_class


News = get_model('news', 'News')


class NewsMenu(CMSAttachMenu):

    name = "News menu"

    def get_nodes(self, request):

        nodes = []
        for entry in News.objects.active():
            node = NavigationNode(
                entry.title,
                entry.get_absolute_url(),
                attr={
                    'description': entry.lead
                })
            nodes.append(node)

        return nodes


menu_pool.register_menu(get_class('news.cms_menus', 'NewsMenu'))



# -*- coding: utf-8 -*-
import operator
import types
from django.conf import settings
from cms.models.pagemodel import Page
from easy_thumbnails.files import get_thumbnailer
from easy_thumbnails.exceptions import InvalidImageFormatError

from allink_core.apps.config.constants import PAGE_FIELD_FALLBACK_CONF
from allink_core.core.loading import get_model

Config = get_model('config', 'Config')
AllinkPageExtension = get_model('config', 'AllinkPageExtension')
AllinkTitleExtension = get_model('config', 'AllinkTitleExtension')


def get_page_meta_dict(page):
    """
    Function which returns all relevant meta fields of a CMSPage. (Defines the fallback hierarchy.)
    Is the Page equivalent of AllinkMetaTagMixin.

    Used in combination with:
    AllinkPageExtension
    AllinkTitleExtension
    """
    allink_config = Config.get_solo()
    meta_context = {
        'meta_page_title': get_meta_page_title(page),
        'meta_title': get_fallback(page, 'meta_title'),
        'meta_description': get_fallback(page, 'meta_description'),
        'meta_image_url': getattr(get_meta_page_image_thumb(page), 'url', ''),
        'google_site_verification': allink_config.google_site_verification,
    }
    return meta_context


def get_meta_page_title(page):
    allink_config = Config.get_solo()
    base_title = ' | ' + getattr(allink_config, 'default_base_title', '')
    page_title = get_fallback(page, 'meta_title')

    return page_title + base_title


def get_meta_page_image_thumb(page):
    """
    :param page:
    cms page object
    :return
    a thumbnail image object
    """
    meta_image = get_fallback(page, 'meta_image')
    return generate_meta_image_thumb(meta_image)


def get_page_teaser_dict(page):
    """
    Function which returns all relevant teaser fields of a CMSPage. (Defines the fallback hierarchy.)
    Is the Page equivalent of AllinkTeaserMixin.

    Used in combination with:
    AllinkPageExtension
    AllinkTitleExtension
    """
    meta_context = {
        'teaser_title': get_fallback(page, 'teaser_title'),
        'teaser_technical_title': get_fallback(page, 'teaser_technical_title'),
        'teaser_description': get_fallback(page, 'teaser_description'),
        'teaser_image': get_fallback(page, 'teaser_image'),
        'teaser_link_text': get_fallback(page, 'teaser_link_text'),
        'teaser_link': page.get_absolute_url(),
    }
    return meta_context


def get_fallback(obj, field_name):
    """
    returns the fallback value of a predefined field for a property (e.g 'meta_image' or 'teaser_title')

    This is either used for CMS Page objects or objects which implement a 'field_fallback_conf' property.

    :param obj:
    either a CMS Page object or an object which subclasses  ...
    :param field_name:
    the key in the .._FIELD_FALLBACK_CONF dict
    :return:
    the value of the fallback field or None (it's your responsibility to handle None values in the template or in a
    _dict mapper method e.g meta_dict or teaser_dict)
    """

    if isinstance(obj, Page):
        conf = PAGE_FIELD_FALLBACK_CONF.get(field_name, list())
        obj_mapper = {
            'self': obj,
            'cms.Title': obj.get_title_obj(),
            'config.Config': get_model('config', 'Config').get_solo(),
            'settings': settings,
        }
    else:
        conf = obj.field_fallback_conf.get(field_name, list())
        obj_mapper = {
            'self': obj,
            'config.Config': get_model('config', 'Config').get_solo(),
            'settings': settings,
        }

    for field_dict in conf:
        model = field_dict.get('model')
        field_lookup = field_dict.get('field')

        obj = obj_mapper.get(model)

        try:
            attr = operator.attrgetter(field_lookup)(obj)
            if attr:
                if isinstance(attr, types.MethodType) and attr():
                    return attr()
                else:
                    return attr
        except AttributeError:
            continue
    else:
        # if there is no fallback listed in ..FALLBACK_CONF for this field, try to return the attribute directly
        getattr(obj, field_name, None)


def generate_meta_image_thumb(meta_image):
    """
    We always want the meta image with a maximum width of 1200px and a height of 630px.
    This is what Facebook recommends. As for other platforms this might not be the best ratio.

    :param meta_image:
    an filer image object
    :return
    a thumbnail image object
    """

    if not meta_image:
        return None

    width = 1200
    height = 630

    thumbnail_options = {'crop': 'smart', 'upscale': True, 'HIGH_RESOLUTION': False,
                         'subject_location': meta_image.subject_location, 'size': (width, height)}

    try:
        thumbnailer = get_thumbnailer(meta_image)
        thumbnail = thumbnailer.get_thumbnail(thumbnail_options)
    except InvalidImageFormatError:
        thumbnail = None
    return thumbnail



# Generated by Django 2.1.8 on 2019-07-02 17:18

import cms.models.fields
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkLegacyLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link_object_id', models.IntegerField(help_text='To which object directs the url.', null=True)),
                ('link_model', models.CharField(help_text='Dotted Path to referenced Model', max_length=300, null=True)),
                ('link_url_name', models.CharField(help_text='Name of the App-URL to use.', max_length=64, null=True)),
                ('link_url_kwargs', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, help_text='Keyword arguments used to reverse url.', null=True, size=None)),
                ('old', models.CharField(help_text='We strip away the anchor part of the URL as this part is not passed to the server.', max_length=255, unique=True, verbose_name='Old Link')),
                ('overwrite', models.CharField(blank=True, help_text="Overwrites 'New Page', use for special urls that are not listed there", max_length=255, null=True, verbose_name='Overwrite Link')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('match_subpages', models.BooleanField(default=False, help_text='If True, matches all subpages and redirects them to this link.', verbose_name='Match subpages')),
                ('last_test_result', models.NullBooleanField(default=None, help_text='Was the last automatic test successfull? (True = Yes, False = No, None = Not yet tested)', verbose_name='Result of last test')),
                ('last_test_date', models.DateTimeField(blank=True, null=True, verbose_name='Date of last test')),
                ('redirect_when_logged_out', models.BooleanField(default=False, help_text='If True, current site will not redirect when user is logged in. If False, the page will be redirected.', verbose_name='Redirect when logged out')),
                ('language', models.CharField(choices=[('en', 'English')], default=('en', 'English'), max_length=200, null=True, verbose_name='Language')),
                ('link_apphook_page', cms.models.fields.PageField(help_text='If provided, overrides the external link.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_legacy_redirect_allinklegacylink_app_legacy_redirects', to='cms.Page', verbose_name='New Apphook-Page')),
                ('link_page', cms.models.fields.PageField(help_text='If provided, overrides the external link and New Apphook-Page.', null=True, on_delete=django.db.models.deletion.PROTECT, to='cms.Page', verbose_name='New Page')),
            ],
            options={
                'verbose_name': 'Legacy Link',
                'verbose_name_plural': 'Legacy Links',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import random
import string

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.test import RequestFactory
from django.contrib.auth.models import User


class CategoryTestCaseMixin:
    """Mixin class for testing Categories"""

    @staticmethod
    def reload(node):
        """NOTE: django-treebeard requires nodes to be reloaded via the Django
        ORM once its sub-tree is modified for the API to work properly.

        See:: https://tabo.pe/projects/django-treebeard/docs/2.0/caveats.html

        This is a simple helper-method to do that."""
        return node.__class__.objects.get(id=node.id)

    @classmethod
    def rand_str(cls, prefix='', length=23, chars=string.ascii_letters):
        return prefix +''.join(random.choice(chars) for _ in range(length))

    @classmethod
    def create_user(cls):
        return User.objects.create(
            username=cls.rand_str(), first_name=cls.rand_str(),
            last_name=cls.rand_str())

    @staticmethod
    def get_request(language=None):
        """
        Returns a Request instance populated with cms specific attributes.
        """
        request_factory = RequestFactory(HTTP_HOST=settings.ALLOWED_HOSTS[0])
        request = request_factory.get("/")
        request.session = {}
        request.LANGUAGE_CODE = language or settings.LANGUAGE_CODE
        # Needed for plugin rendering.
        request.current_page = None
        request.user = AnonymousUser()
        return request



# -*- coding: utf-8 -*-
"""
use core:
no 'urls.py' file needed

!important!
- make sure you also add the new urls to the cms_apps.py so djangocms knows about them! override the apphook,
see example in cms_apps.py
- make sure you respect the patterns in 'allink_core.apps.dummy_app.urls'
"""
from django.urls import path
from .views import DummyAppSomeNewView

urlpatterns = [
    path('new/some-new-view/', DummyAppSomeNewView.as_view(), name='some-new-view'),
]



# -*- coding: utf-8 -*-

__all__ = [
    'check_fallback_conf'
]


def check_fallback_conf():
    """
    The get_fallback function fails silently and returns None if no appropriate fallback field is found.
    However it might be interesting to see when the _FIELD_FALLBACK_CONF is not matching the actual attributes on
    the class.

    This helper checks whether all fallback fields are actually on the class which you are trying to implement.
    (e.g. META_FIELD_FALLBACK_CONF, TEASER_FIELD_FALLBACK_CONF)

    :return:
    a list of exceptions which were thrown
    """
    # TODO implementation
    pass



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.template import Library

register = Library()


@register.inclusion_tag("templatetags/allink_google_tag_manager.html")
def google_tag_manager(tag_id=''):
    if not tag_id and hasattr(settings, 'GOOGLE_TAG_MANAGER_ID'):
        tag_id = settings.GOOGLE_TAG_MANAGER_ID
    return {
        'tag_id': tag_id
    }



# -*- coding: utf-8 -*-
import requests
from requests.exceptions import ConnectionError, RequestException

from django.contrib import messages
from django.conf import settings

from django.db import models
from django.utils.timezone import now
from django.utils.translation import activate, deactivate
from allink_core.core.utils import base_url
from allink_core.core_apps.allink_legacy_redirect.utils import strip_anchor_part
from allink_core.core.models import AllinkInternalLinkFieldsModel


class AllinkLegacyLink(AllinkInternalLinkFieldsModel):
    old = models.CharField('Old Link', max_length=255, unique=True,
                           help_text='We strip away the anchor part of the URL as '
                                     'this part is not passed to the server.')

    #  External Redirect
    overwrite = models.CharField(
        'Overwrite Link',
        max_length=255,
        null=True,
        blank=True,
        help_text='Overwrites \'New Page\', use for special urls that are not listed there'
    )

    active = models.BooleanField(
        'Active',
        default=True,
    )
    match_subpages = models.BooleanField(
        'Match subpages',
        default=False,
        help_text='If True, matches all subpages and redirects them to this link.'
    )
    last_test_result = models.NullBooleanField(
        'Result of last test',
        default=None,
        help_text='Was the last automatic test successfull? (True = Yes, False = No, None = Not yet tested)'
    )
    last_test_date = models.DateTimeField(
        'Date of last test',
        null=True,
        blank=True
    )
    redirect_when_logged_out = models.BooleanField(
        'Redirect when logged out',
        default=False,
        help_text=('If True, current site will not redirect when user is logged in. '
                   'If False, the page will be redirected.')
    )
    language = models.CharField(
        'Language',
        max_length=200,
        null=True
    )

    class Meta:
        verbose_name = 'Legacy Link'
        verbose_name_plural = 'Legacy Links'

    def __str__(self):
        return self.old

    def save(self, *args, **kwargs):
        self.old = strip_anchor_part(self.old)
        super(AllinkLegacyLink, self).save(*args, **kwargs)

    @property
    def link(self):
        if self.overwrite:
            return self.overwrite
        else:
            if self.language:
                activate(self.language)
                link = super(AllinkLegacyLink, self).link
                deactivate()
                return link

    def test_redirect(self, request):
        result = False
        old_url = base_url() + self.old
        new_path = self.link
        new_url = base_url() + '/' + new_path
        try:
            resp = requests.get(old_url)
        except ConnectionError:
            messages.add_message(
                request,
                messages.ERROR,
               'Can\'t connect to url: {}'.format(old_url)
            )
            result = None
        except RequestException as e:
            messages.add_message(request, messages.ERROR, '%s: %s' % (e, self.old))
            result = None
        else:
            # successfull request, test for history
            if resp.status_code == 200:
                try:
                    # first history entry should be the redirect
                    redir = resp.history[0]
                    if redir.status_code == 302:
                        location = redir.headers.get('Location')
                        # location of redirect has to match
                        # Django 1.9 will send back relative urls
                        # if run via dev server
                        # Both absolute and relative urls should
                        # be accounted for
                        # https://en.wikipedia.org/wiki/HTTP_location

                        if location == new_url or location == new_path:
                            result = True
                except IndexError:
                    pass
        self.last_test_result = result
        self.last_test_date = now()
        self.save()
        return result



# -*- coding: utf-8 -*-
from django import forms
from cms.plugin_base import CMSPluginBase
from webpack_loader.utils import get_files

from cms.plugin_pool import plugin_pool

from allink_core.core.utils import get_ratio_choices_orig, get_additional_choices, \
    get_project_color_choices, get_image_width_alias_choices
from allink_core.core_apps.allink_image.models import AllinkImagePlugin
from allink_core.core.forms.fields import ColorField
from allink_core.core.forms.fields import SelectLinkField
from allink_core.core.forms.mixins import AllinkInternalLinkFieldMixin
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AllinkImagePluginForm(AllinkInternalLinkFieldMixin, forms.ModelForm):

    internal_link = SelectLinkField(label='Link Internal', required=False)

    class Meta:
        model = AllinkImagePlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')
        widgets = {
            'caption_text': forms.Textarea(attrs={'rows': 2}),
        }

    def __init__(self, *args, **kwargs):
        super(AllinkImagePluginForm, self).__init__(*args, **kwargs)
        self.fields['link_special'] = forms.CharField(
            label='Special Links',
            widget=forms.Select(choices=self.instance.get_link_special_choices()),
            required=False,
            help_text=('Important: In case the selected option is a <strong>form</strong>, make sure to '
                       'select <strong>Lightbox (Forms)</strong> from the <strong>link target</strong> options '
                       'for best user experience.'),
        )
        self.fields['ratio'] = forms.CharField(
            label='Ratio',
            help_text=('This option overrides the default image ratio set for images in a colum of a '
                       'content section.'),
            widget=forms.Select(choices=get_ratio_choices_orig()),
            required=False,
        )
        if get_image_width_alias_choices():
            self.fields['width_alias'] = forms.CharField(
                label='Width Alias',
                help_text=('This option overrides the default image width_alias set for images in a column of a '
                           'content section.'),
                widget=forms.Select(choices=get_image_width_alias_choices()),
                required=False,
            )
        if get_project_color_choices():
            self.fields['bg_color'] = ColorField(
                label='Set a predefined background color',
                required=False,
            )

        if get_additional_choices('IMAGE_CSS_CLASSES'):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predifined variations',
                choices=get_additional_choices('IMAGE_CSS_CLASSES'),
                required=False,
            )


@plugin_pool.register_plugin
class CMSAllinkImagePlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkImagePlugin
    name = 'Image'
    module = 'Generic'
    form = AllinkImagePluginForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (None, {
                'fields': [
                    'picture',
                    'ratio',
                    'project_css_classes',
                    'icon_enabled',
                    'bg_enabled',

                ]
            }),
            ('Additional settings', {
                'classes': ('collapse',),
                'fields': [
                    'caption_text',
                    'attributes',
                ]
            }),
            ('Link settings', {
                'classes': ('collapse',),
                'fields': (
                    'internal_link',
                    'link_url',
                    ('link_mailto', 'link_phone'),
                    ('link_anchor', 'link_special'),
                    'link_file',
                    'link_target',
                )
            }),
        ]

        if get_project_color_choices():
            fieldsets[0][1].get('fields').append('bg_color')
        if get_image_width_alias_choices():
            fieldsets[1][1].get('fields').append('width_alias')
        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_image/content.html'
        return template

    @classmethod
    def get_render_queryset(cls):
        return cls.model._default_manager.all()



# -*- coding: utf-8 -*-
"""
use core:
from allink_core.apps.dummy_app.admin import *  # noqa
"""
from allink_core.apps.dummy_app.admin import *  # noqa

admin.site.unregister(DummyApp)


@admin.register(DummyApp)
class DummyAppAdmin(DummyAppAdmin):
    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'status',
                    'title',
                    'slug',
                    'entry_date',
                    'template',
                    'preview_image',
                    'lead',
                    'some_field',
                    'some_field_translated_field',

                )
            }),
        )

        fieldsets += (
            ('Published From/To', {
                'classes': ('collapse',),
                'fields': (
                    'start',
                    'end',
                )
            }),
        )

        fieldsets += self.get_category_fieldsets()
        fieldsets += self.get_teaser_fieldsets()
        fieldsets += self.get_seo_fieldsets()
        return fieldsets



# -*- coding: utf-8 -*-
from django import forms
from django.forms.widgets import Media, TextInput
from django.contrib.postgres.forms import SplitArrayField
from django.templatetags.static import static
from django.core.exceptions import ValidationError

from djangocms_attributes_field.widgets import AttributesWidget
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from allink_core.core_apps.allink_button_link.models import AllinkButtonLinkContainerPlugin, AllinkButtonLinkPlugin
from allink_core.core_apps.allink_button_link import widgets
from allink_core.core.utils import get_additional_choices, update_context_google_tag_manager, get_ratio_choices
from allink_core.core.models.choices import (
    BLANK_CHOICE, NEW_WINDOW, SOFTPAGE,
    FORM_MODAL, IMAGE_MODAL, DEFAULT_MODAL, BUTTON_CONTEXT_CHOICES, CONTEXT_DEFAULT, )
from allink_core.core.forms.fields import SelectLinkField
from allink_core.core.forms.mixins import AllinkInternalLinkFieldMixin
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AllinkButtonLinkContainerPluginForm(forms.ModelForm):
    class Meta:
        model = AllinkButtonLinkContainerPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def __init__(self, *args, **kwargs):
        super(AllinkButtonLinkContainerPluginForm, self).__init__(*args, **kwargs)
        if get_additional_choices('BUTTON_LINK_CSS_CLASSES'):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predifined variations',
                choices=get_additional_choices('BUTTON_LINK_CSS_CLASSES'),
                required=False,
            )


class AllinkButtonLinkPluginForm(AllinkInternalLinkFieldMixin, forms.ModelForm):
    LINK_TARGET_REDUCED = (
        (NEW_WINDOW, 'New window'),
        (SOFTPAGE, 'Softpage'),
    )

    link_target_reduced = forms.ChoiceField(label='Link Target', required=False,
                                            choices=BLANK_CHOICE + LINK_TARGET_REDUCED)

    internal_link = SelectLinkField(label='Link Internal', required=False)
    internal_email_addresses = SplitArrayField(forms.EmailField(required=False), size=3, required=False)
    btn_context = forms.ChoiceField(label='Variation', choices=BUTTON_CONTEXT_CHOICES, widget=widgets.Context,
                                    initial=CONTEXT_DEFAULT)

    class Meta:
        model = AllinkButtonLinkPlugin
        exclude = (
            'page', 'position', 'placeholder', 'language', 'plugin_type',
        )
        # When used inside djangocms-text-ckeditor
        # this causes the label field to be prefilled with the selected text.
        widgets = {
            'label': TextInput(attrs={'class': 'js-ckeditor-use-selected-text'}),
        }

    def __init__(self, *args, **kwargs):
        super(AllinkButtonLinkPluginForm, self).__init__(*args, **kwargs)
        self.fields['link_attributes'].widget = AttributesWidget()
        self.fields['link_special'] = forms.CharField(
            label='Special Links',
            widget=forms.Select(choices=self.instance.get_link_special_choices()),
            required=False,
            help_text=('Important: In case the selected option is a <strong>form</strong>, '
                       'make sure to select <strong>Lightbox (Forms)</strong> '
                       'from the <strong>link target</strong> options for best user experience.'),
        )
        self.fields['ratio'] = forms.CharField(
            label='Ratio',
            help_text='This option overrides the default ratio setting for embeded videos.',
            widget=forms.Select(choices=get_ratio_choices()),
            required=False,
        )
        self.initial['link_target_reduced'] = self.instance.link_target

    def _get_media(self):
        """
        Provide a description of all media required to render the widgets on this form
        """
        media = Media()
        for field in self.fields.values():
            media = media + field.widget.media
        return media + Media(js=['cms/js/libs/jquery.min.js'])

    media = property(_get_media)

    def cleanup_default_link(self, cleaned_data):
        cleaned_data['internal_link'] = None
        cleaned_data['link_page'] = None
        cleaned_data['link_apphook_page'] = None
        cleaned_data['link_object_id'] = None
        cleaned_data['link_model'] = None
        cleaned_data['link_url_name'] = None
        cleaned_data['link_url_kwargs'] = None
        cleaned_data['link_url'] = ''
        cleaned_data['link_anchor'] = ''
        cleaned_data['link_target'] = None
        cleaned_data['link_target_reduced'] = None
        return cleaned_data

    def cleanup_form_link(self, cleaned_data):
        cleaned_data['link_special'] = None
        cleaned_data['send_internal_mail'] = True
        cleaned_data['internal_email_addresses'] = None
        cleaned_data['from_email_address'] = None
        cleaned_data['send_external_mail'] = True
        cleaned_data['thank_you_text'] = None
        cleaned_data['label_layout'] = 'stacked'
        return cleaned_data

    def cleanup_file_link(self, cleaned_data):
        template = cleaned_data.get("template")
        if template != AllinkButtonLinkPlugin.IMAGE_LINK:
            cleaned_data['link_file'] = None
        return cleaned_data

    def cleanup_image_link(self, cleaned_data):
        template = cleaned_data.get("template")
        if template != AllinkButtonLinkPlugin.FILE_LINK:
            cleaned_data['link_file'] = None
        return cleaned_data

    def cleanup_video_embedded_link(self, cleaned_data):
        cleaned_data['video_id'] = None
        cleaned_data['video_service'] = None
        cleaned_data['ratio'] = None
        cleaned_data['auto_start_enabled'] = False
        cleaned_data['allow_fullscreen_enabled'] = True
        return cleaned_data

    def cleanup_video_file_link(self, cleaned_data):
        cleaned_data['video_file'] = None
        cleaned_data['video_poster_image'] = None
        cleaned_data['auto_start_enabled'] = False
        cleaned_data['video_muted_enabled'] = True
        return cleaned_data

    def cleanup_email_link(self, cleaned_data):
        cleaned_data['link_mailto'] = None
        cleaned_data['email_subject'] = None
        cleaned_data['email_body_text'] = None
        return cleaned_data

    def cleanup_phone_link(self, cleaned_data):
        cleaned_data['link_phone'] = None
        return cleaned_data

    def validate_form_link(self, cleaned_data):
        """
        If template is a form which sends emails all the additional fields have to be supplied
        """

        if cleaned_data.get('send_internal_mail') is True:
            # if not cleaned_data.get('from_email_address'):
            #     self.add_error('from_email_address', ValidationError('Please supply an E-Mail Address.'))
            for i in range(3):
                if not cleaned_data.get('internal_email_addresses')[i]:
                    self.add_error(
                        'internal_email_addresses', ValidationError('Please supply at least one E-Mail Address.'))
                    break

        elif cleaned_data.get('send_external_mail') is True:
            if not cleaned_data.get('from_email_address'):
                self.add_error('from_email_address', ValidationError('Please supply an E-Mail Address.'))

    def clean(self):
        cleaned_data = super(AllinkButtonLinkPluginForm, self).clean()
        template = cleaned_data.get("template")
        old_template = self.initial.get('template')

        cleanup_template_mapper = {
            AllinkButtonLinkPlugin.DEFAULT_LINK: self.cleanup_default_link,
            AllinkButtonLinkPlugin.FORM_LINK: self.cleanup_form_link,
            AllinkButtonLinkPlugin.FILE_LINK: self.cleanup_file_link,
            AllinkButtonLinkPlugin.IMAGE_LINK: self.cleanup_image_link,
            AllinkButtonLinkPlugin.VIDEO_EMBEDDED_LINK: self.cleanup_video_embedded_link,
            AllinkButtonLinkPlugin.VIDEO_FILE_LINK: self.cleanup_video_file_link,
            AllinkButtonLinkPlugin.EMAIL_LINK: self.cleanup_email_link,
            AllinkButtonLinkPlugin.PHONE_LINK: self.cleanup_phone_link,
        }

        # validation_mapper = {
        #     AllinkButtonLinkPlugin.FORM_LINK: self.validate_form_link,
        # }

        link_target_mapper = {
            # double check for empty string
            AllinkButtonLinkPlugin.DEFAULT_LINK: cleaned_data.get('link_target_reduced', None) or None,
            AllinkButtonLinkPlugin.FORM_LINK: FORM_MODAL,
            AllinkButtonLinkPlugin.FILE_LINK: NEW_WINDOW,
            AllinkButtonLinkPlugin.IMAGE_LINK: IMAGE_MODAL,
            AllinkButtonLinkPlugin.VIDEO_EMBEDDED_LINK: DEFAULT_MODAL,
            AllinkButtonLinkPlugin.VIDEO_FILE_LINK: DEFAULT_MODAL,
        }

        # if template has changed, delete all obsolete fields
        if old_template and template != old_template:
            cleanup_template_mapper.get(old_template)(cleaned_data)

        # validate different link types
        # validation_mapper.get(template)()

        # TODO we make this nicer when we add tests

        # always open external_links in new tab
        if cleaned_data['link_url']:
            cleaned_data['link_target'] = NEW_WINDOW

        cleaned_data['link_target'] = link_target_mapper.get(template)

        return cleaned_data


@plugin_pool.register_plugin
class CMSAllinkButtonLinkContainerPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkButtonLinkContainerPlugin
    name = 'Button/ Link Container'
    module = 'Generic'
    allow_children = True
    child_classes = ['CMSAllinkButtonLinkPlugin']
    form = AllinkButtonLinkContainerPluginForm

    fieldsets = (
        (None, {
            'fields': (
                'alignment_horizontal_desktop',
                'alignment_horizontal_mobile',
            ),
        }),
        ('Advanced settings', {
            'classes': ('collapse',),
            'fields': (
                'project_css_classes',
            )
        }),
    )

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_button_link/content.html'
        return template


@plugin_pool.register_plugin
class CMSAllinkButtonLinkPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkButtonLinkPlugin
    name = 'Button/ Link'
    module = 'Generic'
    allow_children = False
    form = AllinkButtonLinkPluginForm
    change_form_template = 'admin/allink_button_link/change_form.html'
    render_template = 'allink_button_link/item.html'
    text_enabled = True

    fieldsets = (
        (None, {
            'fields': (
                'template',
                'label',
            ),
        }),
        ('Link style', {
            'classes': (
                'collapse',
            ),
            'fields': (
                'type',
                'btn_context',
                'btn_size',
            )
        }),
        ('Internal/External link settings', {
            'classes': (
                'only_when_default_link',
            ),
            'fields': (
                'internal_link',
                'link_url',
                'link_anchor',
                'link_target_reduced',
            )
        }),
        ('File link settings', {
            'classes': (
                'only_when_file_link',
                'only_when_image_link',
            ),
            'fields': (
                'link_file',
            )
        }),
        ('Phone link settings', {
            'classes': (
                'only_when_phone_link',
            ),
            'fields': (
                'link_phone',
            )
        }),
        ('Email link settings', {
            'classes': (
                'only_when_email_link',
            ),
            'fields': (
                'link_mailto',
                'email_subject',
                'email_body_text',
            )
        }),
        ('Form link settings', {
            'classes': (
                'only_when_form_link',
            ),
            'fields': (
                'link_special',
                'send_internal_mail',
                'internal_email_addresses',
                'from_email_address',
                'send_external_mail',
                'thank_you_text',
                'label_layout',
            )
        }),
        ('Video (Embedded) link settings', {
            'classes': (
                'only_when_video_embedded_link',
            ),
            'fields': (
                'video_id',
                'video_service',
                'ratio',
                'auto_start_enabled',
                'allow_fullscreen_enabled',
            )
        }),
        ('Video (File) link settings', {
            'classes': (
                'only_when_video_file_link',
            ),
            'fields': (
                'video_file',
                'video_poster_image',
                'auto_start_enabled',
                'video_muted_enabled',
            )
        }),
        ('Modal Closing options', {
            'classes': (
                'collapse',
                'only_when_image_link',
                'only_when_form_link',
                'only_when_video_embedded_link',
                'only_when_video_file_link',
            ),
            'fields': (
                'data_modal_escape_close_enabled',
                'data_modal_overlay_close_enabled',
                'data_modal_button_close_enabled',
            )
        }),
        ('Advanced settings', {
            'classes': ('collapse',),
            'fields': (
                'link_attributes',
            )
        }),
        ('Hidden settings', {
            'classes': ('hidden',),
            'fields': (
                'link_target',
            )
        }),
    )

    @classmethod
    def get_render_queryset(cls):
        return cls.model._default_manager.all()

    def icon_src(self, instance):
        return static('aldryn_bootstrap3/img/type/button.png')

    def get_render_template(self, context, instance, placeholder):

        template = 'allink_button_link/item.html'
        return template

    def render(self, context, instance, placeholder):
        context = super(CMSAllinkButtonLinkPlugin, self).render(context, instance, placeholder)

        # set svg icons
        if instance.softpage_enabled:
            context.update({'icon': 'softpage'})
        if instance.new_window_enabled:
            context.update({'icon': 'arrow-external'})

        if instance.page:
            context = update_context_google_tag_manager(
                page_name=instance.page.__str__(),
                page_id=instance.page.id,
                plugin_id=instance.id,
                name=instance.label,
                context=context
            )
        else:
            try:
                context = update_context_google_tag_manager(
                    page_name=instance.placeholder.slot,
                    page_id=instance.placeholder.id,
                    plugin_id=instance.id,
                    name=instance.label,
                    context=context
                )
            except AttributeError:
                context = update_context_google_tag_manager(
                    plugin_id=instance.id,
                    name=instance.label,
                    context=context
                )
        return context



# -*- coding: utf-8 -*-
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin
from allink_core.core.loading import get_model

Config = get_model('config', 'Config')
Locations = get_model('locations', 'Locations')


class LocationsToolbar(AllinkBaseModifierMixin, CMSToolbar):
    model = Locations
    app_label = Locations._meta.app_label


toolbar_pool.register(LocationsToolbar)



from allink_core.core.admin.forms import AllinkCategoryAdminForm


__all__ = [
    'AllinkMediaAdminMixin',
    'AllinkSEOAdminMixin',
    'AllinkCategoryAdminMixin',
    'AllinkTeaserAdminMixin',
]


class AllinkMediaAdminMixin:
    """
    ModelAdmin mixin used to add custom css and js
    """

    class Media:
        from webpack_loader.utils import get_files
        js = (
            get_files('djangocms_custom_admin')[1]['publicPath'],
        )
        css = {
            'all': (
                get_files('djangocms_custom_admin')[0]['publicPath'],

            )
        }


class AllinkSEOAdminMixin:
    """
    ModelAdmin mixin used in combination with AllinkSEOFieldsModel and AllinkSEOTranslatedFieldsModel
    """

    def get_seo_fieldsets(self):
        fieldsets = (
            ('SEO', {
                'classes': (
                    'collapse',
                ),
                'fields': (
                    'og_image',
                    'og_title',
                    'og_description',
                )
            }),
        )

        return fieldsets


class AllinkCategoryAdminMixin:
    """
    ModelAdmin mixin used in combination with AllinCategoryFieldsModel

    add:
    list_display = (... 'all_categories_column', ...)

    add:
    list_filter = (
        'status',
        ...
        ('categories', admin.RelatedOnlyFieldListFilter,),
    )
    """

    form = AllinkCategoryAdminForm

    def all_categories_column(self, object):
        return "\n|\n".join([c.name for c in object.categories.all()])

    all_categories_column.short_description = 'Categories'

    def get_category_fieldsets(self):
        fieldsets = ()
        if self.model.get_can_have_categories():
            fieldsets += (
                ('Categories', {
                    'fields': (
                        'categories',
                    )
                }),
            )
        return fieldsets


class AllinkTeaserAdminMixin:
    """
    ModelAdmin mixin used in combination with AllinkTeaserTranslatedFieldsModel
    """

    def get_teaser_fieldsets(self):
        fieldsets = (
            ('Teaser', {
                'classes': (
                    'collapse',
                ),
                'fields': (
                    'teaser_image',
                    'teaser_title',
                    'teaser_technical_title',
                    'teaser_description',
                    'teaser_link_text',
                )
            }),
        )

        return fieldsets



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag
def settings_value(name):
    return getattr(settings, name, "")


@register.simple_tag
def var(val=None):
    return val


@register.filter
def add_params(value, arg):
    """ Used in connection with Django's add_preserved_filters

        Usage:
        - url|add_params:'foo=bar&yolo=no'
        - url|add_params:'foo=bar'|add_params:'motto=yolo'

        use only for admin view
    """
    first = '?'
    if '?' in value:
        first = '&'

    return value + first + str(arg)


@register.simple_tag
def build_param(value, arg):
    """ Used in connection with Django's add_preserved_filters

        Usage:
        - {% build_param 'lang' 'de' as lang_param %}
        - {% build_param 'template' 17 as templ_param %}
        - {% build_param 'template' <var_name> as templ_param %}

        use only for admin view
    """
    return value + '=' + str(arg)



# -*- coding: utf-8 -*-
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin
from .models import DummyApp


class DummyAppToolbar(AllinkBaseModifierMixin, CMSToolbar):
    model = DummyApp
    app_label = DummyApp._meta.app_label


toolbar_pool.register(DummyAppToolbar)



# Generated by Django 2.1.10 on 2019-10-02 19:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_button_link', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkbuttonlinkplugin',
            name='link_target',
            field=models.IntegerField(blank=True, choices=[(1, 'New window'), (2, 'Softpage'), (4, 'Lightbox (Forms)'), (5, 'Lightbox (Image)'), (6, 'Lightbox (Default)')], null=True, verbose_name='Link Target'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.utils import six

from allink_core.core.customisation.utils import create_new_app


class Command(BaseCommand):
    help = ("Create a new app with some allink boilerplate code.")

    def add_arguments(self, parser):
        parser.add_argument(
            'dummy_app',
            help='The module to copy the files from',
            nargs='?',
            default='allink_core.core.customisation.dummy_new_app'
        )
        parser.add_argument('app_label', help='The application name')
        parser.add_argument('model_name', help='The model name')
        parser.add_argument('app_path', help='The path to copy the files to')

    def handle(self, *args, **options):

        dummy_app = options['dummy_app']
        app_label = options['app_label']
        model_name = options['model_name']
        app_path = options['app_path']

        # We usually do not want to call our model Apps. (this was a common mistake)
        if model_name == 'apps':
            self.stdout.write(self.style.ERROR('You probably reversed the order of the arguments. '
                                               'Correct order would be "./manage.py new_app app_label model_name'
                                               ' app_path"'))
            return

        try:
            create_new_app(
                dummy_app=dummy_app,
                app_label=app_label,
                app_path=app_path,
                model_name=model_name
            )
        except Exception as e:
            raise CommandError(six.text_type(e))

        msg = (
            """
            The final steps:\n
            1. add '{app_path}.{app_label}', to PROJECT_APPS \n
            2. add 'CMS{model_name}AppContentPlugin', Plugins to ALLINK_CONTENT_PLUGIN_CHILD_CLASSES \n
            3. add ('{model_name}Apphook', {{'detail': ('apps.{app_label}.models.{model_name}', ['slug'])}}), to PROJECT_LINK_APPHOOKS \n
            4. create a new tuple {model_name_upper}_PLUGIN_TEMPLATES and add all templates from templates/ dir \n
            5. (optional): add ('{model_name_lower}', '{model_name}'), to PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES if the app should have categories\n
            6. ./manage.py makemigrations {app_label} ./manage.py migrate\n
            """  # noqa
        ).format(
            app_label=app_label,
            app_path=app_path,
            model_name=model_name,
            model_name_upper=model_name.upper(),
            model_name_lower=model_name.lower(),
        )
        self.stdout.write(self.style.SUCCESS(msg))



from django.contrib.sitemaps import Sitemap
from django.utils.translation import override as force_language
from cms.sitemaps import CMSSitemap
from allink_core.core.utils import base_url


class CMSHrefLangSitemap(CMSSitemap):
    """
    Base Sitemap class used for djangocms Page objects to provide a proper hreflang tag for each language

    further information: https://support.google.com/webmasters/answer/189077?hl=de

    """

    def _urls(self, page, protocol, domain):
        urls = super()._urls(page, protocol, domain)
        for url in urls:
            url['hreflang'] = []
            title = url.get('item')
            for lang in title.page.languages.split(','):
                if title.page.is_published(lang):
                    with force_language(lang):
                        url['hreflang'].append({
                            'lang': lang,
                            'href': '{}{}'.format(base_url(),
                                                  title.page.get_absolute_url(language=lang, fallback=False))
                        })
        return urls


class HrefLangSitemap(Sitemap):
    """
    Base Sitemap class used in allink apps to provide a proper hreflang tag for each language

    further information: https://support.google.com/webmasters/answer/189077?hl=de

    """

    def _urls(self, page, protocol, domain):
        urls = super()._urls(page, protocol, domain)
        for url in urls:
            url['hreflang'] = []
            item = url.get('item')
            for lang in list(item.get_available_languages()):
                url['hreflang'].append({
                    'lang': lang,
                    'href': '{}{}'.format(base_url(), item.get_absolute_url(language=lang))
                })
                url['active'] = item.status

        return urls



# Generated by Django 2.1.10 on 2019-09-03 17:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('locations', '0002_auto_20190716_0951'),
    ]

    operations = [
        migrations.AddField(
            model_name='locations',
            name='commercial_register_entry',
            field=models.CharField(blank=True, max_length=50, null=True, verbose_name='Commercial register entry'),
        ),
    ]



# -*- coding: utf-8 -*-
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool

class LocationsApphook(CMSApp):
    name = "Locations Apphook"
    app_name = 'locations'

    def get_urls(self, page=None, language=None, **kwargs):
        return ['allink_core.apps.locations.urls']


apphook_pool.register(LocationsApphook)



# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.postgres.fields import ArrayField
from django.utils.functional import cached_property

from cms.models.pluginmodel import CMSPlugin
from filer.fields.image import FilerImageField
from filer.fields.folder import FilerFolderField
from djangocms_text_ckeditor.fields import HTMLField

from allink_core.core.utils import get_additional_templates


class AllinkGalleryPlugin(CMSPlugin):

    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50
    )
    ratio = models.CharField(
        'Ratio',
        max_length=50,
        blank=True,
        null=True
    )
    fullscreen_enabled = models.BooleanField(
        'Fullscreen option visible',
        default=True,
        help_text='This option enables a fullscreen button for this gallery.',
    )
    counter_enabled = models.BooleanField(
        'Gallery counter visible',
        default=False,
        help_text='This option enables a gallery counter.',
    )
    auto_start_enabled = models.BooleanField(
        'Autostart',
        default=False,
        help_text='This option enables autoplay for this gallery.',
    )
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )
    folder = FilerFolderField(
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        help_text=("All Images (.png, .gif, .jpg, .jpeg) will be used in gallery. "
                   "If a folder is specified, the child plugin won't be rendered."),
    )

    def __str__(self):
        if self.template:
            return self.template
        return str(self.pk)

    @classmethod
    def get_templates(cls):
        templates = ()
        for x, y in get_additional_templates('GALLERY'):
            templates += ((x, y),)
        return templates

    @property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        css_classes.append("counter-enabled") if self.counter_enabled else None
        return ' '.join(css_classes)

    @cached_property
    def folder_images(self):
        filename_extensions = ['png', 'gif', 'jpg', 'jpeg']
        images = []
        for image in self.folder.files:
            if image.extension in filename_extensions:
                images.append(image)
        return images

    def save(self, *args, **kwargs):
        super(AllinkGalleryPlugin, self).save(*args, **kwargs)
        AllinkGalleryImagePlugin.objects.filter(parent_id=self.id).update(template=self.template, ratio=self.ratio)


class AllinkGalleryImagePlugin(CMSPlugin):
    title = models.CharField(
        'Title',
        max_length=255,
        blank=True,
        null=True
    )
    text = HTMLField(
        'Text',
        blank=True,
        null=True
    )
    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50,
    )
    ratio = models.CharField(
        'Ratio',
        max_length=50,
        blank=True,
        null=True
    )
    image = FilerImageField(
        verbose_name='Image',
        on_delete=models.PROTECT,
    )

    def __str__(self):
        return'{}'.format(self.image)

    def save(self, *args, **kwargs):
        self.template = self.parent.allink_gallery_allinkgalleryplugin.template
        self.ratio = self.parent.allink_gallery_allinkgalleryplugin.ratio
        super(AllinkGalleryImagePlugin, self).save(*args, **kwargs)



import sys
import random
import string
from django.utils.translation import override
from django.conf import settings
from django.core.cache import cache
from django.urls import clear_url_caches
from django.test.client import RequestFactory
from parler.utils.context import switch_language
from cms.apphook_pool import apphook_pool
from cms.appresolver import clear_app_resolvers
from cms import api
from cms.exceptions import AppAlreadyRegistered
from allink_core.core_apps.allink_categories.models import AllinkCategory

__all__ = [
    'PageApphookMixin',
    'CategoriesMixin',
    'DataModelMixin',
    'DataModelTranslationMixin',
    'GenericPluginMixin',
    'AllinkAppContenPluginMixin',
]


class PageApphookMixin:
    """
    Creates the following objects:

    root_page
    page
    plugin_page
    placeholder

    Cleans up cache and a bunch of cms related stuff.

    You dont need to specify an apphook, but if you want to, for example to test views of apps which require one.
    Just override these attributes:

    apphook_object = NewsApphook
    apphook = 'NewsApphook'
    namespace = 'news'

    To override the cms page template:

    page_template = 'default.html'

    """
    apphook_object = None
    apphook = None
    namespace = None

    page_template = 'default.html'

    def setUp(self):
        super().setUp()
        self.language = settings.LANGUAGES[0][0]
        self.reload_urls()

        self.root_page = api.create_page(
            title='root_page',
            template=self.page_template,
            language=self.language,
            published=True,
        )
        try:
            # Django-cms 3.5 doesn't set is_home when create_page is called
            self.root_page.set_as_homepage()
        except AttributeError:
            pass

        self.page = api.create_page(
            title='page',
            template=self.page_template,
            language=self.language,
            published=True,
            parent=self.root_page,
            apphook=self.apphook,
            apphook_namespace=self.namespace
        )
        self.child_page = api.create_page(
            title='child_page',
            template=self.page_template,
            language=self.language,
            published=True,
            parent=self.page,
        )
        self.plugin_page = api.create_page(
            title="plugin_page",
            template=self.page_template,
            language=self.language,
            parent=self.root_page,
            published=True
        )

        self.placeholder = self.page.placeholders.all()[0]

        # translated and publish all created pages
        for page in self.root_page, self.page, self.child_page, self.plugin_page:
            for language, _ in settings.LANGUAGES[1:]:
                api.create_title(language, '{}: {}'.format(page.get_slug(), language), page)

                # apparently djangocms does not update the title path when create_title is called
                page._update_title_path(language)
                page._update_title_path_recursive(language)

                page.publish(language)
                page.set_translations_cache()

    def tearDown(self):
        """
        Do a proper cleanup, delete everything what is preventing us from
        clean environment for tests.
        :return: None
        """
        self.reset_all()
        cache.clear()
        super(PageApphookMixin, self).tearDown()

    def reset_all(self):
        """
        Reset all that could leak from previous test to current/next test.
        :return: None
        """
        self.delete_app_module()
        self.reload_urls()
        self.apphook_clear()

    def delete_app_module(self):
        """
        Remove APP_MODULE from sys.modules. Taken from cms.
        :return: None
        """
        if self.apphook_object:
            app_module = getattr(self.apphook_object, '__module__')

            if app_module in sys.modules:
                del sys.modules[app_module]

    def apphook_clear(self):
        """
        Clean up apphook_pool and sys.modules. Taken from cms with slight
        adjustments and fixes.
        :return: None
        """
        try:
            apphooks = apphook_pool.get_apphooks()
        except AppAlreadyRegistered:
            # there is an issue with discover apps, or i'm using it wrong.
            # setting discovered to True solves it. Maybe that is due to import
            # from aldryn_events.cms_apps which registers EventListAppHook
            apphook_pool.discovered = True
            apphooks = apphook_pool.get_apphooks()

        for name, label in list(apphooks):
            if apphook_pool.apps[name].__class__.__module__ in sys.modules:
                del sys.modules[apphook_pool.apps[name].__class__.__module__]
        apphook_pool.clear()

    def reload_urls(self):
        """
        Clean up url related things (caches, app resolvers, modules).
        Taken from cms.
        :return: None
        """

        clear_app_resolvers()
        clear_url_caches()

        url_modules = [
            'cms.urls',
            settings.ROOT_URLCONF
        ]

        if self.apphook_object:
            app_module = self.apphook_object.__module__
            package = app_module.split('.')[0]
            url_modules.append('{0}.urls'.format(package))

        for module in url_modules:
            if module in sys.modules:
                del sys.modules[module]


class CategoriesMixin:
    """
    Creates the following objects and its translation:

    category_root
    category_1
    category_2

    """

    def setUp(self):
        super().setUp()
        self.setup_categories()

    @staticmethod
    def reload(node):
        """NOTE: django-treebeard requires nodes to be reloaded via the Django
        ORM once its sub-tree is modified for the API to work properly.
        See:: https://tabo.pe/projects/django-treebeard/docs/2.0/caveats.html
        This is a simple helper-method to do that."""
        return node.__class__.objects.get(id=node.id)

    @classmethod
    def rand_str(cls, prefix=u'', length=23, chars=string.ascii_letters):
        return prefix + u''.join(random.choice(chars) for _ in range(length))

    def setup_categories(self):
        """
        Sets-up i18n categories (self.category_root, self.category_1 and
        self.category_2) for use in tests
        """
        self.language = settings.LANGUAGES[0][0]

        categories = []
        # Set the default language, create the objects
        with override(self.language):
            code = "{0}-".format(self.language)
            self.category_root = AllinkCategory.add_root(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category_root)
            self.category_1 = self.category_root.add_child(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category_1)
            self.category_2 = self.category_root.add_child(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category_2)

        # We should reload category_root, since we modified its children.
        self.category_root = self.reload(self.category_root)

        # Setup the other language(s) translations for the categories
        for language, _ in settings.LANGUAGES[1:]:
            for category in categories:
                with switch_language(category, language):
                    code = "{0}-".format(language)
                    category.name = self.rand_str(prefix=code, length=8)
                    category.save()


class DataModelMixin:
    """
    Creates the following objects:

    allink_config
    entry_1
    entry_2
    entry_3
    entry_4
    entry_5

    usage example attributes:

    data_model_factory = NewsFactory

    """
    data_model_factory = None

    def setUp(self):
        from allink_core.apps.config.tests.factories import ConfigFactory
        super().setUp()
        self.allink_config = ConfigFactory()
        self.entry_1 = self.data_model_factory()
        self.entry_2 = self.data_model_factory()
        self.entry_3 = self.data_model_factory()
        self.entry_4 = self.data_model_factory()
        self.entry_5 = self.data_model_factory()


class DataModelTranslationMixin(DataModelMixin):
    """
    Creates the translations in languages de and fr for the following objects:

    entry_1
    entry_2
    entry_3
    entry_4
    entry_5

    """

    def setUp(self):
        super().setUp()
        entries = [self.entry_1, self.entry_2, self.entry_3, self.entry_4, self.entry_5]

        # Setup the other language(s) translations for the entries
        for language, _ in settings.LANGUAGES[1:]:
            for entry in entries:
                title = '{}-{}'.format(entry.title, language)
                with switch_language(entry, language):
                    entry.title = title
                    entry.save()


class GenericPluginMixin(PageApphookMixin):
    """
    Creates the following objects:

    placeholder
    plugin_model_instance

    usage example attributes:

    plugin_class = CMSNewsAppContentPlugin
    init_kwargs = {template: 'grid_static'}

    """
    plugin_class = None
    init_kwargs = {}

    def setUp(self):
        super().setUp()
        self.placeholder = self.plugin_page.placeholders.all()[0]
        self.init_kwargs = self.get_init_kwargs()

        self.plugin_model_instance = api.add_plugin(
            self.placeholder,
            self.plugin_class,
            self.language,
            **self.init_kwargs
        )

    def get_init_kwargs(self):
        additional_kwargs = {}
        return {**self.init_kwargs, **additional_kwargs}


class AllinkAppContenPluginMixin(GenericPluginMixin):
    """
    Used to test AllinkAppContenPlugin plugins.

    usage example attributes:

    load_more_view = NewsPluginLoadMore

    """

    load_more_view = None
    init_kwargs = {'template': 'grid_static'}

    def get_load_more_view(self):
        """ returns a pseudo instantiated load_more_view """
        load_more_view = self.load_more_view()
        load_more_view.plugin = self.plugin_model_instance
        load_more_view.request = RequestFactory()
        load_more_view.request.GET = {}
        return load_more_view



# -*- coding: utf-8 -*-
from datetime import datetime
from django.db.models import Q
from allink_core.core.models.managers import AllinkBaseModelQuerySet


class AllinkNewsQuerySet(AllinkBaseModelQuerySet):
    def active(self):
        """
        :return:
        all entries which are active today
        """
        today = datetime.today()
        return self.published(start=today, end=today).filter(status=1)

    def latest(self):
        return self.active()\
            .order_by('entry_date', 'id')\
            .distinct('entry_date', 'id')

    def earliest(self):
        return self.active()\
            .order_by('-entry_date', 'id')\
            .distinct('entry_date', 'id')

    def published_fields_empty(self):
        return self.filter(Q(start__isnull=True) & Q(end__isnull=True))

    def published_with_startdate(self, start):
        return self.filter(Q(start__lte=start) & Q(end__isnull=True))

    def published_with_enddate(self, end):
        return self.filter(Q(start__isnull=True) & Q(end__gte=end))

    def published(self, start, end):
        return self.filter(Q(start__lte=start) & Q(end__gte=end)) | self.published_with_startdate(
            start) | self.published_with_enddate(end) | self.published_fields_empty()


AllinkNewsManager = AllinkNewsQuerySet.as_manager



# -*- coding: utf-8 -*-
from cms.plugin_pool import plugin_pool
from allink_core.core.cms_plugins import CMSAllinkBaseAppContentPlugin

from allink_core.core.loading import get_model


LocationsAppContentPlugin = get_model('locations', 'LocationsAppContentPlugin')


@plugin_pool.register_plugin
class CMSLocationsAppContentPlugin(CMSAllinkBaseAppContentPlugin):
    """
    model:
    - where to store plugin instances

    name:
    - name of the plugin
    """
    model = LocationsAppContentPlugin
    name = model.data_model._meta.verbose_name_plural

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(CMSLocationsAppContentPlugin, self).get_fieldsets(request, obj=None)
        fieldsets += ('Map Options', {
            'classes': (
                'collapse',
                'only_when_map',
                'only_when_details_and_map',
            ),
            'fields': (
                'zoom_level',
            )
        }),
        return fieldsets



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.utils import six

from allink_core.core.customisation.utils import fork_allink_app


class Command(BaseCommand):
    help = ("Create a local version of an app in 'allink_core.apps'.")

    def add_arguments(self, parser):
        parser.add_argument(
            'dummy_app',
            help='The module to copy the files from',
            nargs='?',
            default='allink_core.core.customisation.dummy_fork_app_minimum'
        )
        parser.add_argument('app_label', help="The application name of the app in 'allink_core.apps'")
        parser.add_argument('app_path', help='The path to copy the files to')
        parser.add_argument(
            '-e',
            '--example',
            action='store_true',
            help='Creates all the files which can be overwritten including a sample implementation.',
        )

    def handle(self, *args, **options):

        dummy_app = options['dummy_app']
        app_label = options['app_label']
        app_path = options['app_path']
        example = options['example']

        if example:
            dummy_app = 'allink_core.core.customisation.dummy_fork_app_example'

        try:
            fork_allink_app(
                dummy_app=dummy_app,
                app_label=app_label,
                app_path=app_path
            )
        except Exception as e:
            raise CommandError(six.text_type(e))

        msg = (
            """
            The final steps:\n
            1. add '{app_path}.{app_label}', to OVERRIDDEN_ALLINK_CORE_APPS \n
            2. if you have already overwritten templates in the project templates dir, move them to {app_path}/{app_label}/templates/{app_label} \n
            """  # noqa
        ).format(
            app_label=app_label,
            app_path=app_path,
        )

        if example:
            msg += (
                """
                3. make sure you remove all files which you don't need! (be generouse here, you can always have peek into version control later.)\n
                4. make sure you remove all comments which you don't need! (be generouse here, you can always have peek into version control later.)\n
                """  # noqa
            )
        self.stdout.write(self.style.SUCCESS(msg))



# -*- coding: utf-8 -*-
import json

from django import forms
from django.core.cache import cache
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from cms.apphook_pool import apphook_pool
from cms.models import Page
import sortedm2m.fields

from allink_core.core.loading import get_model
from allink_core.core.forms import widgets
from allink_core.core.utils import get_project_color_choices


class Classes(forms.fields.CharField):
    widget = forms.widgets.Textarea


class Icon(forms.fields.CharField):
    widget = widgets.Icon
    DEFAULT = ''

    def __init__(self, *args, **kwargs):
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Icon, self).__init__(*args, **kwargs)


class ZipCode(forms.fields.IntegerField):
    widget = forms.widgets.NumberInput(attrs={'maxlength': 4})


class ColorField(forms.fields.CharField):

    def __init__(self, *args, **kwargs):
        super(ColorField, self).__init__(*args, **kwargs)
        default = None
        for key, val in get_project_color_choices().items():
            if val == self.initial:
                default = key
                break
        self.widget = widgets.SpectrumColorPicker(default=default)

    def clean(self, value):
        if value:
            if len(value) == 4:
                value = '#%s%s%s%s%s%s' % (value[1], value[1], value[2], value[2], value[3], value[3])
            try:
                value = get_project_color_choices()[value]
            except KeyError:
                raise ValidationError(_('Please choose a predefined color.'))
        value = self.to_python(value)
        self.validate(value)
        self.run_validators(value)
        return value


# in progress
class SelectLinkField(forms.fields.ChoiceField):

    def __init__(self, required=True, widget=None, label=None,
                 initial=None, help_text='', *args, **kwargs):
        super(forms.fields.ChoiceField, self).__init__(
            required=required, widget=widgets.SearchSelectWidget(), label=label, initial=initial,
            help_text=help_text, *args, **kwargs
        )
        self.choices = []

    def get_page_and_app_choices(self):
        link_apphooks = settings.PROJECT_LINK_APPHOOKS
        """
        Structure in Settings:
         OrderedDict([
        ('Page', []),
        ('NewsApphook', [{'detail': ('allink_apps.news.models.News', ['slug'])}]),
        .... ])
        """
        cached_choices = cache.get('page_app_link_choices', None)
        if cached_choices:
            return cached_choices

        choices = [(None, '---------')]
        set_cache = True
        for apphook, url_names in link_apphooks.items():
            if apphook == 'Page':
                subchoices = []
                subchoices += [(json.dumps({'page_id': p.id}), '%s %s' % ((p.node.depth - 1) * '---', p))
                               for p in Page.objects.filter(publisher_is_draft=False).order_by('node__path')]
                choices.append(('%s' % 'pages'.upper(), subchoices))

            else:
                try:
                    subchoices = []
                    for p in Page.objects.filter(application_urls=apphook, publisher_is_draft=False):
                        for url_name, info in url_names.items():
                            get_model(info[0].split('.')[-3], info[0].split('.')[-1])
                            obj_model = get_model(info[0].split('.')[-3], info[0].split('.')[-1])
                            subchoices += [(json.dumps({'link_apphook_page_id': p.id, 'link_url_name': url_name,
                                                        'link_object_id': obj.id, 'link_url_kwargs': info[1],
                                                        'link_model': info[0]}), '%s (%s)'
                                            % (obj, p.application_namespace))
                                           for obj in obj_model.objects.all()]
                    choices.append((('%s' % apphook_pool.apps[apphook].app_name.upper()), subchoices))
                # on app load time
                except KeyError:
                    set_cache = False

        if set_cache:
            cache.set('page_app_link_choices', choices, 30)
        return choices


class SortedM2MFormField(sortedm2m.fields.SortedMultipleChoiceField):
    """
    Copied from https://github.com/divio/aldryn-common

    Copied because this package is no longer maintained.
    https://github.com/divio/aldryn-common/commit/6582754be67390d056d4aaa1f799c183e808c914

    """
    widget = widgets.SortedM2MWidget

    def __init__(self, *args, **kwargs):
        kwargs['widget'] = self.widget
        super(SortedM2MFormField, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


class DummyAppApphook(CMSApp):
    name = _("DummyApp Apphook")
    app_name = 'dummy_app'

    def get_urls(self, page=None, language=None, **kwargs):
        return ['apps.dummy_app.urls']


apphook_pool.register(DummyAppApphook)



# -*- coding: utf-8 -*-
import re
from os.path import splitext
from django.db.models import Q
from allink_core.core.constants import STOP_WORDS_RE
from django.conf import settings
from django.utils.text import get_valid_filename as get_valid_filename_django
from django.template.defaultfilters import slugify
_base_url = None


def base_url():
    from django.contrib.sites.models import Site
    global _base_url
    if not '_base_url' not in locals() or not _base_url:
        scheme = 'http://' if settings.STAGE == 'local' else 'https://'
        _base_url = scheme + Site.objects.get_current().domain
    return _base_url


def get_additional_templates(model_name):
    """
    Get additional templates choices from settings for model_name
    """
    config = '{}_PLUGIN_TEMPLATES'.format(model_name.upper())
    additional_templates = getattr(settings, config, ())
    return additional_templates


def get_project_css_classes(model_name):
    """
    Get css_classes defined in settings for model_name
    """
    config = '{}_CSS_CLASSES'.format(model_name.upper())
    project_css_classes = getattr(settings, config, ())
    return project_css_classes


def get_additional_choices(config, blank=False):
    """
    Get additional settings var for project specific configuration
    """
    from allink_core.core.models.choices import BLANK_CHOICE
    choices = ()
    if blank:
        choices = BLANK_CHOICE
    choices += getattr(settings, config, ())
    return choices


def get_project_color_choices():
    """
    returns all projects specific colors
    """
    return getattr(settings, 'PROJECT_COLORS', {})


def get_ratio_choices():
    """
    returns all projects specific ratio choices
    """
    from allink_core.core.models.choices import BLANK_CHOICE, RATIO_CHOICES
    return BLANK_CHOICE + RATIO_CHOICES + get_additional_choices('RATIO_CHOICES')


def get_ratio_choices_orig():
    """
    returns all projects specific ratio choices
    including a 'original', which keeps the one from the file itself
    """
    from allink_core.core.models.choices import BLANK_CHOICE, RATIO_CHOICES_ORIG
    return BLANK_CHOICE + RATIO_CHOICES_ORIG + get_additional_choices('RATIO_CHOICES')


def get_image_width_alias_choices():
    """
    returns all projects specific ratio choices
    including a 'original', which keeps the one from the file itself
    """
    from allink_core.core.models.choices import BLANK_CHOICE, IMAGE_WIDTH_ALIAS_CHOICES
    image_width_alias_choices = get_additional_choices('IMAGE_WIDTH_ALIAS_CHOICES')
    if image_width_alias_choices:
        return BLANK_CHOICE + IMAGE_WIDTH_ALIAS_CHOICES + image_width_alias_choices
    return None


def get_height_from_ratio(width, ratio_w, ratio_h):
    """
    Used to calculate thumbnail height from given width and ratio
    """
    return width * ratio_h / ratio_w


def get_percent(full, value):
    """
    returns a value in percent. (how much percent of ..)
    rounded whole numbers
    """
    return round(value * (100 / full))


def get_ratio_w_h(ratio):
    """
    returns width and height from string e.g. '1-2' or '4-3' as integer
    """
    w, h = ratio.split('-')
    return float(w), float(h)


def get_display(key, list):
    """ returns key from list of tuples """
    d = dict(list)
    try:
        return d.get(int(key))
    except KeyError:
        return None


def replace_line_breaks(string):
    """replaces: '\r\n' and '\n\r' and '\r' and '\n' and with '<br />' """
    r = '<br />'
    return string.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)


def get_valid_filename(s):
    """
    like the regular get_valid_filename, but also slugifies away umlauts and
    stuff. Copied from django-filer
    """
    s = get_valid_filename_django(s)
    filename, ext = splitext(s)
    filename = slugify(filename)
    ext = slugify(ext)
    if ext:
        return "%s.%s" % (filename, ext)
    else:
        return "%s" % (filename,)


def normalize_query(query_string):
    """
    Split the query string into individual keywords, getting rid of unecessary
    spaces and grouping quoted words together.
    """
    find_terms = re.compile(r'"([^"]+)"|(\S+)').findall
    normalize_space = re.compile(r'\s{2,}').sub

    # Split the string into terms.
    terms = find_terms(query_string)

    # Only send unquoted terms through the stop words filter.
    for index, term in enumerate(terms):
        if term[1] != '':
            # If the term is a stop word, delete it from the list.
            if STOP_WORDS_RE.sub('', term[1]) == '':
                del terms[index]

    return [normalize_space(' ', (t[0] or t[1]).strip()) for t in terms]


def get_query(query_string, search_fields):
    """Return a query which is a combination of Q objects."""
    query = None
    terms = normalize_query(query_string)

    for term in terms:
        or_query = None

        for field_name in search_fields:
            q = Q(**{"%s__icontains" % field_name: term})
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q

        if query is None:
            query = or_query
        else:
            query = query & or_query

    return query


def get_key_from_dict(dict, value):
    """Takes a dict and a value, returns a list with matching keys."""
    try:
        return list(dict.keys())[list(dict.values()).index(value)]
    except ValueError:
        return None


def get_all_fields_from_form(instance):
    """"
    Return names of all available fields from given Form instance.

    :arg instance: Form instance
    :returns list of field names
    :rtype: list
    """

    fields = list(instance().base_fields)

    for field in list(instance().declared_fields):
        if field not in fields:
            fields.append(field)
    return fields


def update_context_google_tag_manager(context, page_name='NOPAGE_NAME', page_id='NOPAGE_ID', plugin_id='NOPLUGIN_ID',
                                      name='NONAME'):
    """
    form_name = '{}{}_{}_{}'.format(page_name, page_id, plugin_id, name)
    If we have a form we will compile a id like this id="Kontakt88_plugin7451_SupportView"
    If its a Button Link we will try to compile it like  this id="Kontakt88_plugin7451_Support-Formular"
    If the Button Link Plugin is inside a static placeholder we will use the placeholder.slot and id instead of
    page infos
    """
    form_name = '{}{}_plugin{}_{}'.format(page_name, page_id, plugin_id, name)
    form_name = form_name.replace(' ', '-')
    context.update({'form_name': form_name})
    return context



# # -*- coding: utf-8 -*-
from django.urls import path

from allink_core.core_apps.allink_cms.api import CMSPluginAPIView


# TODO this is only the beginning
# we will add a complete api for pages/ placeholders/ plugins etc. (preferably as json)
app_name = 'allink_cms'

urlpatterns = [
    path('plugins/<int:id>/', CMSPluginAPIView.as_view(), name='plugins'),
]



# -*- coding: utf-8 -*-
"""
use core:
from allink_core.apps.dummy_app.cms_apps import *  # noqa
"""
from allink_core.core.loading import unregister_cms_apps
from allink_core.apps.dummy_app.cms_apps import *  # noqa

unregister_cms_apps(DummyAppApphook)


class DummyAppApphook(DummyAppApphook):
    def get_urls(self, page=None, language=None, **kwargs):
        urls = super().get_urls(**kwargs)
        return urls + ['apps.dummy_app.urls']


apphook_pool.register(DummyAppApphook)



from django.http import HttpResponsePermanentRedirect
from django.db.models import Q
from itertools import combinations, permutations

from allink_core.core_apps.allink_legacy_redirect.models import AllinkLegacyLink


class AllinkLegacyRedirectMiddleware(object):
    """
    If the request matches one of the urls configured in AllinkLegacyLink a HttpResponsePermanentRedirect returned to
    the appropriate url.

    Google Click ids will be preserved.
    """

    def __init__(self, get_response):
        self.get_response = get_response


    def __call__(self, request):  # noqa TODO this needs to be refactored anyway, so we do not hit de db each time.
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        response = self.get_response(request)

        has_get_parameters = False
        link = None
        try:
            link = AllinkLegacyLink.objects.get(Q(old=request.path) | Q(
                old=request.path + '/') | Q(old=request.get_full_path()), active=True)

            # if user is logged in, skip redirect
            if link.redirect_when_logged_out and request.user.is_authenticated:
                return response

        except AllinkLegacyLink.DoesNotExist:
            # Here we handle the case that the old url
            # can have GET parameters, and some of them but not all
            # are used to specify a redirection
            for i in range(len(request.GET) - 1, 0, -1):
                possible_olds = []
                # we need to check all permutations for all subsets
                # of GET parameters. First we check the most
                # specific ones. That means the ones with
                # all but one parameter. (All parameters are
                # already checked with "get_full_path" in the first
                # query on top.)
                combis = combinations(request.GET.items(), i)
                perms = [permutations(combi) for combi in combis]
                for perm in perms:
                    for get_parameters in perm:
                        possible_olds.append(request.path + '?' + '&'.join(['%s=%s' % (param, value)
                                                                            for param, value in get_parameters]))
                        possible_olds.append(request.path + '/?' + '&'.join(['%s=%s' % (param, value)
                                                                             for param, value in get_parameters]))
                    try:
                        link = AllinkLegacyLink.objects.get(old__in=possible_olds, active=True)
                        has_get_parameters = True
                        break
                    except AllinkLegacyLink.DoesNotExist:
                        continue

        if not link:
            # check for links which match subpages
            # this must be done after the GET parameter checks,
            # cause subpages are less specific.

            # build up array with path parts
            # e.g. ['en','agency','contact']
            path_bits = request.path.split('/')[1:-1]
            length = len(path_bits)
            olds = []

            # omit final bit because we already
            # tested for it in our first try
            # e.g. ['/en/','/en/agency/']
            for i in range(1, length):
                olds.append('/%s/' % ('/').join(path_bits[:i]))

            # if we got multiple links with match_subpages
            # enabled (e.g. /en/ and /en/agency/), return
            # the longer one
            link = AllinkLegacyLink.objects.filter(
                old__in=olds,
                match_subpages=True,
                active=True
            ).order_by('old').last()

        # if user is logged in skip, redirect
        if link and link.redirect_when_logged_out and request.user.is_authenticated:
            return response

        if not link:
            return response

        # 'overwrite' takes priority over 'new'
        new_link = link.link
        if not new_link:
            return response

        # preserve Google Click Identifier
        if 'gclid' in request.GET:
            if has_get_parameters:
                new_link += '&gclid=%s' % request.GET['gclid']
            else:
                new_link += '?gclid=%s' % request.GET['gclid']

        return HttpResponsePermanentRedirect(new_link)



import os
import shutil
import importlib
from distutils.dir_util import copy_tree
from distutils.errors import DistutilsFileError
from allink_core import ALLINK_CORE_ALLINK_APPS

__all__ = [
    'get_path',
    'copy_dir',
    'replace_strings',
    'rename',
    'fork_allink_app',
    'create_new_app',
]


def get_path(module_str):
    """
    :param module_str
    the module string e.g 'allink_core.core.customisation.dummy_app'
    :return
    the path to the directory e.g '/usr/local/lib/python3.6/site-packages/allink-core/core/customisation/dummy_app'
    """
    module = importlib.import_module(module_str)
    return os.path.dirname(module.__file__)


def copy_dir(dummy_app, app_path):
    """
    copies a directory to another directory

    :param dummy_app:
    path to directory e.g 'allink_core/core/customisation/dummy_app'
    :param app_path:
    path where the new app should created in
    :return
    destination dir
    """
    if os.path.isdir(app_path):
        raise OSError("Can't create destination directory. '{}' directory already exists!".format(app_path))
    try:
        dst = copy_tree(dummy_app, app_path)
    except (OSError, DistutilsFileError) as e:
        raise e

    return dst


def replace_strings(file_path, replace):
    """
    replace strings in a file

    :param file_path:
    path to file
    :param replace:
    dict of strings to replace. e.g:
    replace = {
        'dummy_app': 'new_app',
        'DummyApp': 'NewApp',
        'dummy-app': 'new-app',
    }
    """
    with open(file_path, 'r') as f:
        s = f.read()
        for item in replace.items():
            s = s.replace(item[0], item[1])

    with open(file_path, 'w') as f:
        f.write(s)


def rename(path, replace):
    """
    renames directory or file
    :param path:
    path to file or dir
    :param replace:
    dict of strings to replace. e.g:
    replace = {
        'dummy_app': 'new_app',
        'dummyapp': 'newapp',
        'dummyapp': model_name.lower(),  # lower model name
    }
    :return:
    new dir file path
    """
    basename = os.path.basename(path)
    if any(sub in basename for sub in replace.keys()):
        for item in filter(lambda i: i[0] in basename, replace.items()):
            new_base = basename.replace(item[0], item[1])
            new_path = os.path.join(os.path.dirname(path), new_base)
            os.replace(path, new_path)
            path = new_path


def rename_and_replace(path, replace):
    """
    renames and replaces strings in:
     - filenames
     - directories
     - inside files

    :param path:
    the path in which all files and dirs should be renamed and replaced
    :param replace:
    a dict containing all the  strings which should be replaced
    """
    for root, dirs, files in os.walk(path):
        for dir in dirs:
            if not dir == '__pycache__':
                # rename directories
                dir_path = os.path.join(root, dir)
                rename(path=dir_path, replace=replace)

    # file renaming needs to be done in a second iteration as the directory names have changed
    for root, dirs, files in os.walk(path):
        for file in files:
            if not file.endswith('.pyc'):
                # rename file names
                file_path = os.path.join(root, file)
                # rename dummy classes and import statements
                replace_strings(file_path=file_path, replace=replace)
                # rename file and dir names
                rename(path=file_path, replace=replace)


def create_new_app(dummy_app, app_path, app_label, model_name):
    """
    copies a dummy_app directory renames and replaces dummy naming

    :param dummy_app:
    path to dummy_app e.g 'allink_core.core.customisation.dummy_app'
    :param app_path:
    path where the new app should created e.g 'apps'
    :param app_label:
    new app directory name e.g 'new_app'
    :param model_name:
    new model name name e.g 'NewApp'
    """
    dummy_app_path = get_path(dummy_app)
    new_app_path = os.path.join(app_path, app_label)

    # copy directory
    copy_dir(dummy_app_path, new_app_path)

    replace = {
        'dummy_app': app_label,  # app label
        'DummyApp': model_name,  # model name
        'dummy-app': app_label.replace('_', '-'),  # css class
        'dummyapp': model_name.lower(),  # lower model name
    }
    rename_and_replace(new_app_path, replace)


def fork_allink_app(dummy_app, app_path, app_label):
    """
    forks an app in alink_core.apps
     - renames and replaces dummy naming
     - copies all existing migrations

    :param dummy_app:
    path to dummy_app e.g 'allink_core.core.customisation.dummy_fork_app'
    :param app_path:
    path where the new app should created e.g 'apps'
    :param app_label:
    app label e.g 'news' needs to be in ALLINK_CORE_ALLINK_APPS
    """
    dummy_app_path = get_path(dummy_app)
    allink_core_app_path = os.path.join(get_path('allink_core.apps'), app_label)
    new_app_path = os.path.join(app_path, app_label)
    model_name = app_label.capitalize()

    if not any(app_label in app.split('.')[-1] for app in ALLINK_CORE_ALLINK_APPS):
        raise ValueError("There is no allink_core app with the app_label '{}'".format(app_label))

    # copy directory
    copy_dir(dummy_app_path, new_app_path)

    replace = {
        'dummy_app': app_label,  # app label
        'DummyApp': model_name,  # model name
        'dummy-app': app_label.replace('_', '-'),  # css class
        'dummyapp': model_name.lower(),  # lower model name
    }
    rename_and_replace(new_app_path, replace)

    # copy migrations from allink_core app_path to the new app_path
    shutil.copytree(
        os.path.join(allink_core_app_path, 'migrations'),
        os.path.join(new_app_path, 'migrations')
    )



# -*- coding: utf-8 -*-
from django import template
from cms.models import Page
from cms.templatetags.cms_tags import CMSEditableObject
from django.utils import translation

register = template.Library()


@register.simple_tag
def page_from_slug(slug):
    return Page.objects.get(slug=slug)


@register.simple_tag
def placeholder_has_content(placeholder):
    """
    Renders boolean to variable if placeholder has plugins or not in active language
    """

    return bool(placeholder.cmsplugin_set.filter(language=translation.get_language()))


class CMSEditableObjectAjax(CMSEditableObject):
    """
    Used to render correct title (with editable double click)
    """
    edit_template_ajax = 'templatetags/allink_plugin_ajax.html'

    def _is_ajax(self, request):
        return request.is_ajax()

    def get_template(self, context, **kwargs):
        if self._is_editable(context.get('request', None)):
            if self._is_ajax(context.get('request', None)):
                return self.edit_template_ajax
            else:
                return self.edit_template
        return self.template


register.tag('render_model_ajax', CMSEditableObjectAjax)



# Generated by Django 2.1.8 on 2019-07-02 17:18

import aldryn_translation_tools.models
import allink_core.core.models.mixins
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import parler.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lft', models.PositiveIntegerField(db_index=True)),
                ('rgt', models.PositiveIntegerField(db_index=True)),
                ('tree_id', models.PositiveIntegerField(db_index=True)),
                ('depth', models.PositiveIntegerField(db_index=True)),
                ('model_names', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=50), blank=True, help_text='Please specify the app which uses this categories. All apps specified in parent category are automatically added.', null=True, size=None)),
                ('tag', models.CharField(blank=True, help_text='auto-generated categories use this tag, to identify which app generated the category.', max_length=80, null=True, verbose_name='Tag')),
                ('identifier', models.CharField(blank=True, help_text='Identifier used for backward reference on a app model. (e.g display category name on People app, e.g Marketing)', max_length=50, null=True, verbose_name='Identifier')),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
            bases=(allink_core.core.models.mixins.AllinkTranslatedAutoSlugifyMixin, aldryn_translation_tools.models.TranslationHelperMixin, parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='AllinkCategoryTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(default='', max_length=255, verbose_name='name')),
                ('slug', models.SlugField(blank=True, default='', help_text='Provide a “slug” or leave blank for an automatically generated one.', max_length=255, verbose_name='slug')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='allink_categories.AllinkCategory')),
            ],
            options={
                'verbose_name': 'Category Translation',
                'db_table': 'allink_categories_allinkcategory_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.AlterUniqueTogether(
            name='allinkcategorytranslation',
            unique_together={('language_code', 'master'), ('language_code', 'slug')},
        ),
    ]



# Generated by Django 2.1.10 on 2019-10-02 19:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0003_auto_20190716_0951'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='newsappcontentplugin',
            name='filter_fields',
        ),
    ]



# -*- coding: utf-8 -*-
"""
use core:
no 'sitemaps.py' file needed
"""
from allink_core.apps.dummy_app.sitemaps import DummyAppSitemap as CoreDummyAppSitemap


class DummyAppSitemap(CoreDummyAppSitemap):
    pass



# -*- coding: utf-8 -*-
import pickle
from django.db import models
from django.core.cache import cache
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.translation import get_language
from cms.extensions import PageExtension, TitleExtension
from menus.menu_pool import menu_pool
from parler.models import TranslatableModel, TranslatedFieldsModel
from parler.utils.context import switch_language
from filer.fields.image import FilerImageField
from solo.models import SingletonModel
from solo import settings as solo_settings
from djangocms_text_ckeditor.fields import HTMLField

from allink_core.core.models import (
    AllinkTeaserFieldsModel,
    AllinkTeaserTranslatedFieldsModel,
    AllinkSEOFieldsModel,
    AllinkSEOTranslatedFieldsModel,
)


class BaseConfig(TranslatableModel, SingletonModel):
    default_og_image = FilerImageField(
        verbose_name='og:image',
        on_delete=models.PROTECT,
        help_text=(
            'Default preview image when page/post is shared on Facebook. <br>Min. 1200 x 630 for best results.'),
        blank=True,
        null=True
    )
    theme_color = models.CharField(
        'Theme Color',
        help_text='Theme color for Android Chrome',
        max_length=50,
        blank=True,
        null=True
    )
    mask_icon_color = models.CharField(
        'Mask icon color',
        help_text='Mask icon color for safari-pinned-tab.svg',
        max_length=50,
        blank=True,
        null=True,
    )
    msapplication_tilecolor = models.CharField(
        'msapplication TileColor',
        help_text='MS application TitleColor Field',
        max_length=50,
        blank=True,
        null=True
    )
    google_site_verification = models.CharField(
        'Google Site Verification Code',
        blank=True,
        null=True,
        max_length=64
    )
    default_to_email = models.EmailField(
        'Default to email',
        default='itcrowd@allink.ch',
        max_length=255
    )
    default_from_email = models.EmailField(
        'Default from email',
        default='itcrowd@allink.ch',
        max_length=255
    )

    newsletter_teaser_counter = models.IntegerField(
        'Newsletter Teaser Counter',
        help_text='Number which defines after how many newsentries the newsletter teaser should appear '
                  '(on grid with newsletterteaser). Default is 4',
        default=4,
    )

    class Meta:
        abstract = True
        app_label = 'config'
        verbose_name = 'Configuration'

    def __str__(self):
        return 'Configuration'

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        from cms.cache import invalidate_cms_page_cache
        invalidate_cms_page_cache()

        # the config is used widely across all apps and cms pages
        cache.clear()

        # invalidate the menu for all sites
        for site in Site.objects.all():
            menu_pool.clear(site_id=site.id)

    def set_to_cache(self):
        """
        saves pickled instance to cache in each language
        """
        for language_code, _ in settings.LANGUAGES:
            cache_key = self.get_cache_key(language_code)
            timeout = getattr(settings, 'SOLO_CACHE_TIMEOUT', solo_settings.SOLO_CACHE_TIMEOUT)
            with switch_language(self, language_code):
                cache.set(cache_key, pickle.dumps(self), timeout)

    @classmethod
    def get_cache_key(cls, language_code):
        """
        e.g: 'solo:config_de'
        """
        return '%s:%s_%s' % ('solo', cls.__name__.lower(), language_code)

    @classmethod
    def get_solo(cls):
        """
        Loads pickled Config instance from cache.
        If it does not already exists in cache, it will get or create it form the database.
        :return:
        Config instance
        """
        cache_key = cls.get_cache_key(get_language())
        obj_dict = cache.get(cache_key)
        if obj_dict:
            obj = pickle.loads(obj_dict)
        if not obj_dict:
            obj, created = cls.objects.get_or_create(pk=cls.singleton_instance_id)
            obj.set_to_cache()
        return obj


class BaseConfigTranslation(TranslatedFieldsModel):
    master = models.ForeignKey(
        'config.Config',
        on_delete=models.CASCADE,
        related_name='translations',
        null=True)

    default_base_title = models.CharField(
        verbose_name='Base title',
        max_length=50,
        help_text=(
            'Default base title, Is also used for default base og:title when page/post is shared on Facebook. <br>'
            'If not supplied the name form Django Sites will be used instead.'),
        blank=True,
        null=True
    )
    newsletter_lead = HTMLField(
        'Newsletter Signup Text',
        help_text='Teaser text in the Newsletter Teaser',
        blank=True,
        null=True,
    )
    newsletter_signup_link = models.URLField(
        'Newsletter Signup Link',
        help_text='Link for Button on Newsletter Teaser',
        blank=True,
        null=True,
    )
    newsletter_declaration_of_consent = HTMLField(
        'Declaration of consent',
        help_text='Detailed declaration of consent.',
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True
        app_label = 'config'


class BaseAllinkPageExtension(AllinkSEOFieldsModel, AllinkTeaserFieldsModel, PageExtension):
    class Meta:
        abstract = True
        app_label = 'config'


class BaseAllinkTitleExtension(AllinkSEOTranslatedFieldsModel, AllinkTeaserTranslatedFieldsModel, TitleExtension):
    class Meta:
        abstract = True
        app_label = 'config'



# -*- coding: utf-8 -*-
"""
use core:
from allink_core.apps.dummy_app.cms_toolbars import *  # noqa
"""
from allink_core.apps.dummy_app.cms_toolbars import *  # noqa

Config = get_model('config', 'Config')

toolbar_pool.unregister(DummyAppToolbar)


class DummyAppToolbar(DummyAppToolbar):
    pass


toolbar_pool.register(DummyAppToolbar)



from django.test.client import RequestFactory
from django.test.testcases import TestCase
from allink_core.core.test import CategoriesMixin, DataModelMixin, AllinkAppContenPluginMixin
from allink_core.apps.news.cms_apps import NewsApphook
from .factories import NewsFactory
from ..cms_plugins import CMSNewsAppContentPlugin


class CMSNewsAppContentPluginTestCaseApp(CategoriesMixin, DataModelMixin, AllinkAppContenPluginMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'
    apphook_object = NewsApphook
    data_model_factory = NewsFactory
    plugin_class = CMSNewsAppContentPlugin

    def test_context_object_list_all(self):
        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory()}
        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertEqual(context['object_list'].count(), 5)
        self.assertNotIn('page_obj', context)
        self.assertNotIn('next_page_url', context)

    def test_context_object_list_manual_entries(self):
        """ TODO: will be obsolete after refactoring, because tested on plugin model """
        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2])
        context = {'request': RequestFactory()}
        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertEqual(context['object_list'].count(), 2)
        self.assertNotIn('page_obj', context)
        self.assertNotIn('next_page_url', context)

    def test_context_object_list_category_navigation_enabled(self):
        """ TODO: will be obsolete after refactoring, because tested on plugin model """
        self.plugin_model_instance.categories.set([self.category_1, self.category_2,])
        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        self.entry_1.categories.set([self.category_1,])
        self.entry_2.categories.set([self.category_1,])
        self.entry_3.categories.set([self.category_1,])
        self.entry_4.categories.set([self.category_2, ])
        self.plugin_model_instance.category_navigation_enabled = True

        context = {'request': RequestFactory()}
        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertEqual(context['object_list'].count(), 3)
        self.assertNotIn('page_obj', context)
        self.assertNotIn('next_page_url', context)

    def test_context_pagination(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD

        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory()}

        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertEqual(context['object_list'].count(), 2)
        self.assertIn('page_obj', context)
        self.assertIn('next_page_url', context)

    def test_context_pagination_no_next_page(self):
        self.plugin_model_instance.paginated_by = 5
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD

        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory()}

        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertEqual(context['object_list'].count(), 5)
        self.assertNotIn('page_obj', context)
        self.assertNotIn('next_page_url', context)

    def test_context_category_navigation(self):
        self.plugin_model_instance.category_navigation_enabled = True

        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory()}

        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertIn('category_navigation', context)
        self.assertIn('by_category', context)

    def test_context_category_navigation_next_page_url_with_manual_entries(self):
        """
        Special case, where we want to have a category navigation (without the tab "All") on manual_entries with a Load More Button.
        """
        self.plugin_model_instance.category_navigation_enabled = True
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2, self.entry_3])
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD

        self.entry_1.categories.set([self.category_1,])
        self.entry_2.categories.set([self.category_1,])
        self.entry_3.categories.set([self.category_1,])
        self.entry_4.categories.set([self.category_2, ])

        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory()}
        context = plugin_instance.render(context, self.plugin_model_instance, None)
        self.assertIn('next_page_url', context)

    def test_get_render_template_no_result(self):
        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory(), 'object_list': []}

        render_template = plugin_instance.get_render_template(context, self.plugin_model_instance, None)
        self.assertIn('no_results.html', render_template)

    def test_get_render_template_results(self):
        plugin_instance = self.plugin_model_instance.get_plugin_class_instance()
        context = {'request': RequestFactory(), 'object_list': [self.entry_1]}

        render_template = plugin_instance.get_render_template(context, self.plugin_model_instance, None)
        self.assertNotIn('no_results.html', render_template)







# Generated by Django 2.1.8 on 2019-07-04 13:38

from django.db import migrations
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='news',
            options={'ordering': ('-entry_date',), 'verbose_name': 'News entry', 'verbose_name_plural': 'News'},
        ),
        migrations.RemoveField(
            model_name='news',
            name='sort_order',
        ),
        migrations.AddField(
            model_name='news',
            name='entry_date',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='Entry Date'),
        ),
    ]



# # -*- coding: utf-8 -*-
from django.urls import path

from allink_core.core.loading import get_class

PeoplePluginLoadMore = get_class('people.views', 'PeoplePluginLoadMore')
PeopleDetail = get_class('people.views', 'PeopleDetail')


urlpatterns = [
    path('<int:page>/', PeoplePluginLoadMore.as_view(), name='more'),
    path('<slug:slug>/', PeopleDetail.as_view(), name='detail'),
]



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings

from easy_thumbnails.files import get_thumbnailer
from easy_thumbnails.exceptions import InvalidImageFormatError

from allink_core.core.loading import get_model
from allink_core.core.utils import get_height_from_ratio, get_ratio_w_h

register = template.Library()

Config = get_model('config', 'Config')


# ####################################################################################
# Allink image

def get_sizes_from_width_alias(width_alias, image):
    sizes = []
    aliases = settings.THUMBNAIL_WIDTH_ALIASES
    for point in aliases.get(width_alias):
        width = aliases.get(width_alias).get(point).get('width')
        ratio = aliases.get(width_alias).get(point).get('ratio')
        # original ratio
        if ratio == 'x-y':
            height = get_height_from_ratio(width, image.width, image.height)
        else:
            ratio_w, ratio_h = get_ratio_w_h(ratio)
            height = get_height_from_ratio(width, ratio_w, ratio_h)
        sizes.append((point, (width, height)))
    return sizes


def get_width_alias_from_plugin(plugin):
    """
    This function is called from different context:

    TODO This would be a lot more explicit, if we would make width_alias a mandatory parameter on render_image and
         evaluate the correct width_alias on a property on the plugin itself. (e.g on AllinkImagePlugin,
         AllinkGalleryPlugin or AllinkBaseAppContentPlugin)

    1. inside a AllinkContentColumnPlugin -> the width_alias corresponding to the parents
       (always a CMSAllinkContentPlugin) template field is returned

    2. AllinkBaseAppContentPlugin -> the width_alias corresponding to the field items_per_row is returned

    3. some other context or without a plugin instance -> always '2-of-3': this is a fallback, but should not come up
       if the correct width_alias is supplied in the template

    :param plugin:
    a model instance of a cms plugin

    :return:
    a string with the appropriate width alias depending on which context the plugin is embedded.
    """
    # plugin directly inside a column plugin
    # if isinstance(plugin, AllinkContentColumnPlugin):
    if not hasattr(plugin, 'items_per_row'):
        # the outer most parent of all plugin with pictures should always be a content/column plugin
        # if not return a fallback of '1-of-1'. (potential_column gets None and will trow a AttributeError)
        try:
            # find the next column plugin
            column_plugin = None
            potential_column = plugin.parent.get_plugin_instance()[0]
            while (column_plugin is None):
                if potential_column.plugin_type == 'CMSAllinkContentColumnPlugin':
                    column_plugin = potential_column
                else:
                    potential_column = potential_column.parent.get_plugin_instance()[0]
            if column_plugin.template == 'col-1':
                return '1-of-1'
            elif column_plugin.template == 'col-1-1':
                return '1-of-2'
            elif column_plugin.template == 'col-1-2':
                return '1-of-3' if column_plugin.position == 0 else '2-of-3'
            elif column_plugin.template == 'col-2-1':
                return '2-of-3' if column_plugin.position == 0 else '1-of-3'
            elif column_plugin.template == 'col-3':
                return '1-of-3'
            elif column_plugin.template == 'col-4':
                return '1-of-4'
            else:
                return '1-of-1'
        except AttributeError:
            return '2-of-3'

    # app plugin
    # if isinstance(plugin, AllinkBaseAppContentPlugin):
    elif hasattr(plugin, 'items_per_row'):
        return '1-of-{}'.format(getattr(plugin, 'items_per_row', '1'))

    # template tag called from within an other context
    # this is a fallback, but should not come up if the correct width_alias is supplied in the template
    else:
        return '2-of-3'


def get_unique_key(context):
    """
    :param context:
    context which is expected to have a plugin instance
    :return:
    either the plugin id
    or if no plugin instance is in context, the id from the object itself (e.g a news entry) and the image id
    """
    if context.get('instance'):
        return context.get('instance').id
    else:
        return '{}-{}'.format(context.get('object').id, context.get('image').id)


@register.inclusion_tag('templatetags/allink_image.html', takes_context=True)
def render_image(context, image, alt_text='', ratio=None, width_alias=None, crop='smart', upscale=True, bw=False,
                 high_resolution=True, icon_enabled=False, bg_enabled=True, bg_color=None, lazyload_enabled=True,
                 zoom=None, subject_location=True, vh_enabled=False):
    """
    -> parameters:
    image: FilerImageField

    width_alias: '1-of-1'  -> needs to be supplied, when outside of the content or app plugin content.

    -> optional parameters:
    ratio: '3-2' -> to explicitly use the original ratio pass in 'x-y'
    alt_text: alternative image alt text
    crop: used for thumbnail gen
    bw: used for thumbnail gen
    icon_disabled: used in template
    bg_disabled: used in template
    bg_color: used in template
    zoom: used in template
    subject_location: used for thumbnail gen
    vh_enabled: used in template (using vh instead of percent)

    if you render a image from outside the content or app plugin content,
    it is important to supply a thumbnail width_alias
    otherwise the thumbnail will be rendered with the default width_alias '1-of-1' which might be to big.

    -> makes following context variable available in the template
    image (original image)
    thumbnail_url_xs
    thumbnail_url_sm
    thumbnail_url_md
    thumbnail_url_lg
    thumbnail_url_xl
    thumbnail_url_xxl
    icon_disabled
    bg_disabled
    bg_color

    """
    if image:
        # explicit render image in this width_alias
        # most likely not from within an app plugin template or a content template
        if not width_alias:
            # get with alias from context
            width_alias = get_width_alias_from_plugin(context.get('instance', None))

        # use focal point. works best in combination with zoom > 0
        if subject_location:
            subject_location = image.subject_location

        # update context
        context.update({'image': image})
        context.update({'icon_enabled': icon_enabled})
        context.update({'bg_enabled': bg_enabled})
        context.update({'bg_color': bg_color})
        context.update({'lazyload_enabled': lazyload_enabled})
        context.update({'alt_text': alt_text})
        context.update({'vh_enabled': vh_enabled})

        sizes = get_sizes_from_width_alias(width_alias, image)
        thumbnail_options = {'crop': crop, 'bw': bw, 'upscale': upscale, 'HIGH_RESOLUTION': high_resolution,
                             'zoom': zoom, 'subject_location': subject_location}

        # create a thumbnail for each size
        for size in sizes:
            thumbnailer = get_thumbnailer(image)

            # override the default ratio or use THUMBNAIL_WIDTH_ALIASES ratio
            if ratio:
                w = size[1][0]
                # original ratio
                if ratio == 'x-y':
                    h = get_height_from_ratio(w, image.width, image.height)
                else:
                    ratio_w, ratio_h = get_ratio_w_h(ratio)
                    h = get_height_from_ratio(w, ratio_w, ratio_h)
            else:
                w, h = size[1][0], size[1][1]

            thumbnail_options.update({'size': (w, h)})
            context.update({'ratio_percent_{}'.format(size[0]): '{}%'.format(h / w * 100)})
            context.update({'ratio_vh_{}'.format(size[0]): '{}vh'.format(h / w * 100)})
            try:
                context.update({'thumbnail_{}'.format(size[0]): thumbnailer.get_thumbnail(thumbnail_options)})
            except InvalidImageFormatError:
                context.update({'thumbnail_{}'.format(size[0]): None})
        # for css padding hack, a image in each ratio has to be unique
        # (break point doesnt matter, because is never shown at the same time)
        context.update({
            'picture_id': 'picture-{}'.format(
                '-'.join((str(image.id), str(get_unique_key(context)), str(round(w)), str(round(h))))
            )})

    return context
